# SPDX-License-Identifier: LGPL-2.1-or-later

# **************************************************************************
#   Copyright (c) 2011 Juergen Riegel <FreeCAD@juergen-riegel.net>        *
#                                                                         *
#   This file is part of the FreeCAD CAx development system.              *
#                                                                         *
#   This program is free software; you can redistribute it and/or modify  *
#   it under the terms of the GNU Lesser General Public License (LGPL)    *
#   as published by the Free Software Foundation; either version 2 of     *
#   the License, or (at your option) any later version.                   *
#   for detail see the LICENCE text file.                                 *
#                                                                         *
#   FreeCAD is distributed in the hope that it will be useful,            *
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#   GNU Library General Public License for more details.                  *
#                                                                         *
#   You should have received a copy of the GNU Library General Public     *
#   License along with FreeCAD; if not, write to the Free Software        *
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#   USA                                                                   *
# **************************************************************************

import FreeCAD, unittest, Part
import copy
import math
from FreeCAD import Units
from FreeCAD import Base

App = FreeCAD

from parttests.BRep_tests import BRepTests
from parttests.Geom2d_tests import Geom2dTests
from parttests.regression_tests import RegressionTests
from parttests.TopoShapeListTest import TopoShapeListTest
from parttests.TopoShapeTest import TopoShapeTest
from parttests.TestPartMirror import TestPartMirroringRegression


# ---------------------------------------------------------------------------
# define the test cases to test the FreeCAD Part module
# ---------------------------------------------------------------------------
def getCoincidentVertexes(vtx1, vtx2):
    pairs = []
    tol = Part.Precision.confusion()
    for i in vtx1:
        for j in vtx2:
            if i.Point.distanceToPoint(j.Point) < tol:
                pairs.append((i, j))

    return pairs


class PartTestCases(unittest.TestCase):
    def setUp(self):
        self.Doc = FreeCAD.newDocument("PartTest")

    def testBoxCase(self):
        self.Box = self.Doc.addObject("Part::Box", "Box")
        self.Doc.recompute()
        self.assertEqual(len(self.Box.Shape.Faces), 6)

    def testIssue2985(self):
        v1 = App.Vector(0.0, 0.0, 0.0)
        v2 = App.Vector(10.0, 0.0, 0.0)
        v3 = App.Vector(10.0, 0.0, 10.0)
        v4 = App.Vector(0.0, 0.0, 10.0)
        edge1 = Part.makeLine(v1, v2)
        edge2 = Part.makeLine(v2, v3)
        edge3 = Part.makeLine(v3, v4)
        edge4 = Part.makeLine(v4, v1)
        # Travis build confirms the crash under macOS
        # result = Part.makeFilledFace([edge1,edge2,edge3,edge4])
        # self.Doc.addObject("Part::Feature","Face").Shape = result
        # self.assertTrue(isinstance(result.Surface, Part.BSplineSurface))

    def tearDown(self):
        # closing doc
        FreeCAD.closeDocument("PartTest")
        # print ("omit closing document for debugging")


class PartTestBSplineCurve(unittest.TestCase):
    def setUp(self):
        self.Doc = FreeCAD.newDocument("PartTest")

        poles = [[0, 0, 0], [1, 1, 0], [2, 0, 0]]
        self.spline = Part.BSplineCurve()
        self.spline.buildFromPoles(poles)

        poles = [[0, 0, 0], [1, 1, 0], [2, 0, 0], [1, -1, 0]]
        self.nurbs = Part.BSplineCurve()
        self.nurbs.buildFromPolesMultsKnots(poles, (3, 1, 3), (0, 0.5, 1), False, 2)

    def testProperties(self):
        self.assertEqual(self.spline.Continuity, "CN")
        self.assertEqual(self.spline.Degree, 2)
        self.assertEqual(self.spline.EndPoint, App.Vector(2, 0, 0))
        self.assertEqual(self.spline.FirstParameter, 0.0)
        self.assertEqual(self.spline.FirstUKnotIndex, 1)
        self.assertEqual(self.spline.KnotSequence, [0.0, 0.0, 0.0, 1.0, 1.0, 1.0])
        self.assertEqual(self.spline.LastParameter, 1.0)
        self.assertEqual(self.spline.LastUKnotIndex, 2)
        max_degree = self.spline.MaxDegree
        self.assertEqual(self.spline.NbKnots, 2)
        self.assertEqual(self.spline.NbPoles, 3)
        self.assertEqual(self.spline.StartPoint, App.Vector(0.0, 0.0, 0.0))

    def testGetters(self):
        """only check if the function doesn't crash"""
        self.spline.getKnot(1)
        self.spline.getKnots()
        self.spline.getMultiplicities()
        self.spline.getMultiplicity(1)
        self.spline.getPole(1)
        self.spline.getPoles()
        self.spline.getPolesAndWeights()
        self.spline.getResolution(0.5)
        self.spline.getWeight(1)
        self.spline.getWeights()

    def testSetters(self):
        spline = copy.copy(self.spline)
        spline.setKnot(1, 0.1)
        spline.setPeriodic()
        spline.setNotPeriodic()
        # spline.setKnots()
        # spline.setOrigin(2)   # not working?
        self.spline.setPole(1, App.Vector([1, 0, 0]))  # first parameter 0 gives occ error

    def testIssue2671(self):
        if "BUILD_SPREADSHEET" in FreeCAD.__cmake__:
            self.Doc = App.newDocument("Issue2671")
            Box = self.Doc.addObject("Part::Box", "Box")
            Mirroring = self.Doc.addObject("Part::Mirroring", "Mirroring")
            Spreadsheet = self.Doc.addObject("Spreadsheet::Sheet", "Spreadsheet")
            Mirroring.Source = Box
            Mirroring.Base = (8, 5, 25)
            Mirroring.Normal = (0.5, 0.2, 0.9)
            Spreadsheet.set("A1", "=Mirroring.Base.x")
            Spreadsheet.set("B1", "=Mirroring.Base.y")
            Spreadsheet.set("C1", "=Mirroring.Base.z")
            Spreadsheet.set("A2", "=Mirroring.Normal.x")
            Spreadsheet.set("B2", "=Mirroring.Normal.y")
            Spreadsheet.set("C2", "=Mirroring.Normal.z")
            self.Doc.recompute()
            self.assertEqual(Spreadsheet.A1, Units.Quantity("8 mm"))
            self.assertEqual(Spreadsheet.B1, Units.Quantity("5 mm"))
            self.assertEqual(Spreadsheet.C1, Units.Quantity("25 mm"))
            self.assertEqual(Spreadsheet.A2, Units.Quantity("0.5 mm"))
            self.assertEqual(Spreadsheet.B2, Units.Quantity("0.2 mm"))
            self.assertEqual(Spreadsheet.C2, Units.Quantity("0.9 mm"))
            App.closeDocument("Issue2671")

    def testIssue2876(self):
        if "BUILD_SPREADSHEET" in FreeCAD.__cmake__:
            self.Doc = App.newDocument("Issue2876")
            Cylinder = self.Doc.addObject("Part::Cylinder", "Cylinder")
            Cylinder.Radius = 5
            Pipe = self.Doc.addObject("Part::Thickness", "Pipe")
            Pipe.Faces = (Cylinder, ["Face2", "Face3"])
            Pipe.Mode = 1
            Pipe.Value = -1  # negative wall thickness
            Spreadsheet = self.Doc.addObject("Spreadsheet::Sheet", "Spreadsheet")
            Spreadsheet.set("A1", "Pipe OD")
            Spreadsheet.set("B1", "Pipe WT")
            Spreadsheet.set("C1", "Pipe ID")
            Spreadsheet.set("A2", "=2*Cylinder.Radius")
            Spreadsheet.set("B2", "=-Pipe.Value")
            Spreadsheet.set("C2", "=2*(Cylinder.Radius + Pipe.Value)")
            self.Doc.recompute()
            self.assertEqual(Spreadsheet.B2, Units.Quantity("1 mm"))
            self.assertEqual(Spreadsheet.C2, Units.Quantity("8 mm"))
            App.closeDocument("Issue2876")

    def testSubElements(self):
        box = Part.makeBox(1, 1, 1)
        with self.assertRaises(ValueError):
            box.getElement("InvalidName")
        with self.assertRaises(ValueError):
            box.getElement("Face6_abc")
        # getSubTopoShape now catches this before it gets to OCC, so the error changes:
        # with self.assertRaises(Part.OCCError):
        with self.assertRaises(IndexError):
            box.getElement("Face7")

    def tearDown(self):
        # closing doc
        FreeCAD.closeDocument("PartTest")


class PartTestCurveToNurbs(unittest.TestCase):
    def testCircleToNurbs(self):
        mat = Base.Matrix()
        mat.rotateX(1)
        mat.rotateY(1)
        mat.rotateZ(1)

        circle = Part.Circle()
        circle.Radius = 5

        circle.transform(mat)
        nurbs = circle.toNurbs()
        self.assertEqual(circle.value(0), nurbs.value(0))

        arc = circle.trim(0, 2)
        nurbs = arc.toNurbs()
        self.assertEqual(circle.value(0), nurbs.value(0))

        spline = circle.toBSpline()
        self.assertAlmostEqual(circle.value(0).distanceToPoint(spline.value(0)), 0)

    def testEllipseToNurbs(self):
        mat = Base.Matrix()
        mat.rotateX(1)
        mat.rotateY(1)
        mat.rotateZ(1)

        ellipse = Part.Ellipse()
        ellipse.MajorRadius = 5
        ellipse.MinorRadius = 3

        ellipse.transform(mat)
        nurbs = ellipse.toNurbs()
        self.assertEqual(ellipse.value(0), nurbs.value(0))

        arc = ellipse.trim(0, 2)
        nurbs = arc.toNurbs()
        self.assertEqual(ellipse.value(0), nurbs.value(0))

        spline = ellipse.toBSpline()
        self.assertAlmostEqual(ellipse.value(0).distanceToPoint(spline.value(0)), 0)


class PartTestBSplineSurface(unittest.TestCase):
    def testTorusToSpline(self):
        to = Part.Toroid()
        bs = to.toBSpline()
        bs.setUPeriodic()
        bs.setVPeriodic()
        self.assertGreater(len(bs.UKnotSequence), 0)
        self.assertGreater(len(bs.VKnotSequence), 0)

    def testBounds(self):
        to = Part.Toroid()
        bs = to.toBSpline()
        self.assertAlmostEqual(bs.bounds()[1], 2 * math.pi)
        self.assertAlmostEqual(bs.bounds()[3], 2 * math.pi)
        bs.scaleKnotsToBounds(0.0, 1.0, 0.0, 1.0)
        self.assertAlmostEqual(bs.bounds()[1], 1.0)
        self.assertAlmostEqual(bs.bounds()[3], 1.0)


class PartTestNormals(unittest.TestCase):
    def setUp(self):
        self.face = Part.makePlane(1, 1)

    def testFaceNormal(self):
        self.assertEqual(self.face.normalAt(0, 0), Base.Vector(0, 0, 1))
        self.assertEqual(self.face.Surface.normal(0, 0), Base.Vector(0, 0, 1))

    def testReverseOrientation(self):
        self.face.reverse()
        self.assertEqual(self.face.normalAt(0, 0), Base.Vector(0, 0, -1))
        self.assertEqual(self.face.Surface.normal(0, 0), Base.Vector(0, 0, 1))

    def testPlacement(self):
        self.face.reverse()
        self.face.Placement.Rotation.Angle = 1
        self.face.Placement.Rotation.Axis = (1, 1, 1)
        vec = Base.Vector(-0.63905, 0.33259, -0.69353)
        self.assertGreater(self.face.normalAt(0, 0).dot(vec), 0.9999)
        self.assertLess(self.face.Surface.normal(0, 0).dot(vec), -0.9999)

    def tearDown(self):
        pass


class PartTestShapeRotate(unittest.TestCase):
    def testPlacement(self):
        tol = 1e-12

        box = Part.makeBox(1, 1, 1)
        box.Placement.Base = Base.Vector(10, 10, 10)
        box.rotate((0, 0, 0), (0, 0, 1), 90)

        p1 = Base.Placement()
        p1.Base = Base.Vector(10, 10, 10)

        p2 = Base.Placement()
        p2.Rotation.Angle = math.radians(90)
        self.assertTrue(box.Placement.isSame(p2 * p1, tol))

        p3 = p1.copy()
        p3.rotate((0, 0, 0), (0, 0, 1), 90)
        self.assertTrue(p3.isSame(p1 * p2, tol))
        self.assertFalse(box.Placement.isSame(p3, tol))

        p4 = p1.copy()
        p4.rotate((0, 0, 0), (0, 0, 1), 90, True)
        self.assertTrue(p4.isSame(p2 * p1, tol))
        self.assertTrue(box.Placement.isSame(p4, tol))


class PartTestCircle2D(unittest.TestCase):
    def testValidCircle(self):
        p1 = App.Base.Vector2d(0.01, 0.01)
        p2 = App.Base.Vector2d(0.02, 0.02)
        p3 = App.Base.Vector2d(0.01, -0.01)
        Part.Geom2d.Circle2d.getCircleCenter(p1, p2, p3)

    def testCollinearPoints(self):
        p1 = App.Base.Vector2d(0.01, 0.01)
        p2 = App.Base.Vector2d(0.02, 0.02)
        p3 = App.Base.Vector2d(0.04, 0.0399)
        with self.assertRaises(ValueError):
            Part.Geom2d.Circle2d.getCircleCenter(p1, p2, p3)


class PartTestCone(unittest.TestCase):
    def testderivatives(self):
        def get_dn(surface, u, v):
            pos = surface.value(u, v)
            v10 = surface.getDN(u, v, 1, 0)
            v01 = surface.getDN(u, v, 0, 1)
            v11 = surface.getDN(u, v, 1, 1)
            return (pos, v10, v01, v11)

        cone = Part.Cone()
        cone.SemiAngle = 0.2
        cone.Radius = 2.0

        u, v = (5.0, 5.0)
        vp, v1, v2, v3 = get_dn(cone, u, v)

        shape = cone.toShape(0, 2 * math.pi, 0, 10)
        shape = shape.toNurbs()
        spline = shape.Face1.Surface

        u, v = spline.parameter(vp)
        wp, w1, w2, w3 = get_dn(spline, u, v)

        self.assertAlmostEqual(vp.distanceToPoint(wp), 0)
        self.assertAlmostEqual(v1.getAngle(w1), 0)
        self.assertAlmostEqual(v2.getAngle(w2), 0)
        self.assertAlmostEqual(v3.getAngle(w3), 0)


class PartTestChFi2dAlgos(unittest.TestCase):
    def testChFi2d_FilletAlgo(self):
        v = FreeCAD.Vector
        edge1 = Part.makeLine(v(0, 0, 0), v(0, 10, 0))
        edge2 = Part.makeLine(v(0, 10, 0), v(10, 10, 0))
        wire = Part.Wire([edge1, edge2])
        pln = Part.Plane()

        with self.assertRaises(TypeError):
            alg = Part.ChFi2d.FilletAlgo(pln)

        alg = Part.ChFi2d.FilletAlgo()
        with self.assertRaises(TypeError):
            alg.init()

        print(alg)
        # Test without shape
        with self.assertRaises(Base.CADKernelError):
            alg.perform(1)

        with self.assertRaises(TypeError):
            alg.perform()

        alg = Part.ChFi2d.FilletAlgo(wire, pln)
        alg.init(edge1, edge2, pln)
        alg.init(wire, pln)

        alg = Part.ChFi2d.FilletAlgo(edge1, edge2, pln)
        alg.perform(1.0)

        with self.assertRaises(TypeError):
            alg.numberOfResults()

        with self.assertRaises(TypeError):
            alg.result(1)

        self.assertEqual(alg.numberOfResults(Base.Vector(0, 10, 0)), 1)
        result = alg.result(Base.Vector(0, 10, 0))
        curve = result[0].Curve
        self.assertEqual(type(curve), Part.Circle)
        self.assertEqual(curve.Axis, pln.Axis)
        self.assertEqual(curve.Radius, 1.0)

    def testChFi2d_AnaFilletAlgo(self):
        v = FreeCAD.Vector
        edge1 = Part.makeLine(v(0, 0, 0), v(0, 10, 0))
        edge2 = Part.makeLine(v(0, 10, 0), v(10, 10, 0))
        wire = Part.Wire([edge1, edge2])
        pln = Part.Plane()

        with self.assertRaises(TypeError):
            alg = Part.ChFi2d.AnaFilletAlgo(pln)

        alg = Part.ChFi2d.AnaFilletAlgo()
        with self.assertRaises(TypeError):
            alg.init()

        print(alg)
        # Test without shape
        self.assertFalse(alg.perform(1))

        with self.assertRaises(TypeError):
            alg.perform()

        alg = Part.ChFi2d.AnaFilletAlgo(wire, pln)
        alg.init(edge1, edge2, pln)
        alg.init(wire, pln)

        alg = Part.ChFi2d.AnaFilletAlgo(edge1, edge2, pln)
        alg.perform(1.0)

        with self.assertRaises(TypeError):
            alg.result(1)

        result = alg.result()
        curve = result[0].Curve
        self.assertEqual(type(curve), Part.Circle)
        self.assertEqual(curve.Radius, 1.0)

    def testChFi2d_ChamferAPI(self):
        v = FreeCAD.Vector
        edge1 = Part.makeLine(v(0, 0, 0), v(0, 10, 0))
        edge2 = Part.makeLine(v(0, 10, 0), v(10, 10, 0))
        wire = Part.Wire([edge1, edge2])

        with self.assertRaises(TypeError):
            alg = Part.ChFi2d.ChamferAPI(edge1)

        alg = Part.ChFi2d.ChamferAPI(wire)
        with self.assertRaises(TypeError):
            alg.init()

        print(alg)

        with self.assertRaises(TypeError):
            alg.perform(1)

        alg = Part.ChFi2d.ChamferAPI(wire)
        alg.init(edge1, edge2)
        alg.init(wire)

        alg = Part.ChFi2d.ChamferAPI(edge1, edge2)
        alg.perform()

        with self.assertRaises(TypeError):
            alg.result(1)

        result = alg.result(1.0, 1.0)
        curve = result[0].Curve
        self.assertEqual(type(curve), Part.Line)


class PartTestRuledSurface(unittest.TestCase):
    def setUp(self):
        self.Doc = FreeCAD.newDocument()

    def testRuledSurfaceFromTwoObjects(self):
        line1 = Part.makeLine(FreeCAD.Vector(-70, -30, 0), FreeCAD.Vector(-50, 40, 0))
        line2 = Part.makeLine(FreeCAD.Vector(-40, -30, 0), FreeCAD.Vector(-40, 10, 0))
        plm1 = FreeCAD.Placement()
        plm1.Rotation = FreeCAD.Rotation(0.7071067811865476, 0.0, 0.0, 0.7071067811865475)
        line1.Placement = plm1
        fea1 = self.Doc.addObject("Part::Feature")
        fea2 = self.Doc.addObject("Part::Feature")
        fea1.Shape = line1
        fea2.Shape = line2
        ruled = self.Doc.addObject("Part::RuledSurface")
        ruled.Curve1 = fea1
        ruled.Curve2 = fea2

        self.Doc.recompute()

        same1 = getCoincidentVertexes(fea1.Shape.Vertexes, ruled.Shape.Vertexes)
        same2 = getCoincidentVertexes(fea2.Shape.Vertexes, ruled.Shape.Vertexes)
        self.assertEqual(len(same1), 2)
        self.assertEqual(len(same2), 2)

    def testRuledSurfaceFromTwoObjectsWithSharedVertexs(self):
        """Test reproducing issue #15539"""

        # Arrange
        line = Part.makeLine(FreeCAD.Vector(0, 0, 50), FreeCAD.Vector(0, -50, 0))
        line1 = Part.makeLine(FreeCAD.Vector(0, 0, 50), FreeCAD.Vector(50, 0, 0))
        line2 = Part.makeLine(FreeCAD.Vector(0, 0, 0), FreeCAD.Vector(0, -50, 0))
        line3 = Part.makeLine(FreeCAD.Vector(0, 0, 0), FreeCAD.Vector(50, 0, 0))
        fea = self.Doc.addObject("Part::Feature")
        fea1 = self.Doc.addObject("Part::Feature")
        fea2 = self.Doc.addObject("Part::Feature")
        fea3 = self.Doc.addObject("Part::Feature")
        fea.Shape = line
        fea1.Shape = line1
        fea2.Shape = line2
        fea3.Shape = line3

        # Defining all the surfaces generated by 2 lines with one common vertex
        ruled = self.Doc.addObject("Part::RuledSurface")
        ruled.Curve1 = fea
        ruled.Curve2 = fea1

        ruled1 = self.Doc.addObject("Part::RuledSurface")
        ruled1.Curve1 = fea1
        ruled1.Curve2 = fea3

        ruled2 = self.Doc.addObject("Part::RuledSurface")
        ruled2.Curve1 = fea2
        ruled2.Curve2 = fea3

        ruled3 = self.Doc.addObject("Part::RuledSurface")
        ruled3.Curve1 = fea
        ruled3.Curve2 = fea2

        # Act
        self.Doc.recompute()

        # Assert
        same00 = getCoincidentVertexes(fea.Shape.Vertexes, ruled.Shape.Vertexes)
        same03 = getCoincidentVertexes(fea.Shape.Vertexes, ruled3.Shape.Vertexes)
        same10 = getCoincidentVertexes(fea1.Shape.Vertexes, ruled.Shape.Vertexes)
        same11 = getCoincidentVertexes(fea1.Shape.Vertexes, ruled1.Shape.Vertexes)
        same22 = getCoincidentVertexes(fea2.Shape.Vertexes, ruled2.Shape.Vertexes)
        same23 = getCoincidentVertexes(fea2.Shape.Vertexes, ruled3.Shape.Vertexes)
        same31 = getCoincidentVertexes(fea3.Shape.Vertexes, ruled1.Shape.Vertexes)
        same32 = getCoincidentVertexes(fea3.Shape.Vertexes, ruled2.Shape.Vertexes)

        # BRepFill::Face() seems creating faces with 4 vertexes
        # Therefore, in this case, every face shares 3 vertexes (1 counted twice as coincident) with the lines that generate that face
        self.assertEqual(len(same00), 3)
        self.assertEqual(len(same03), 3)
        self.assertEqual(len(same10), 3)
        self.assertEqual(len(same11), 3)
        self.assertEqual(len(same22), 3)
        self.assertEqual(len(same23), 3)
        self.assertEqual(len(same31), 3)
        self.assertEqual(len(same32), 3)

    def testRuledSurfaceFromOneObject(self):
        if "BUILD_SKETCHER" in FreeCAD.__cmake__:
            sketch = self.Doc.addObject("Sketcher::SketchObject", "Sketch")
            sketch.Placement = FreeCAD.Placement(
                FreeCAD.Vector(0.000000, 0.000000, 0.000000),
                App.Rotation(0.707107, 0.000000, 0.000000, 0.707107),
            )
            sketch.MapMode = "Deactivated"

            sketch.addGeometry(
                Part.LineSegment(
                    App.Vector(-43.475811, 34.364464, 0), App.Vector(-65.860519, -20.078733, 0)
                ),
                False,
            )
            sketch.addGeometry(
                Part.LineSegment(
                    App.Vector(14.004498, 27.390331, 0), App.Vector(33.577049, -27.952749, 0)
                ),
                False,
            )

            ruled = self.Doc.addObject("Part::RuledSurface", "Ruled Surface")
            ruled.Curve1 = (sketch, ["Edge1"])
            ruled.Curve2 = (sketch, ["Edge2"])
            self.Doc.recompute()

            same = getCoincidentVertexes(sketch.Shape.Vertexes, ruled.Shape.Vertexes)
            self.assertEqual(len(same), 4)

    def tearDown(self):
        FreeCAD.closeDocument(self.Doc.Name)


class PartTestShapeFix(unittest.TestCase):
    def testShapeFix_Root(self):
        with self.assertRaises(TypeError):
            Part.ShapeFix.Root([])

        fix = Part.ShapeFix.Root()
        print(fix)

        fix.Precision = 0.0
        self.assertEqual(fix.Precision, 0.0)

        fix.MinTolerance = 0.0
        self.assertEqual(fix.MinTolerance, 0.0)

        fix.MaxTolerance = 0.5
        self.assertEqual(fix.MaxTolerance, 0.5)

        self.assertEqual(fix.limitTolerance(0.25), 0.25)

    def testShapeFix_Shape(self):
        surface = Part.Plane()
        face = surface.toShape(-1, 1, -1, 1)

        with self.assertRaises(TypeError):
            Part.ShapeFix.Shape([])

        fix = Part.ShapeFix.Shape(face)
        fix.init(face)
        print(fix)
        fix.shape()
        fix.fixSolidTool()
        fix.fixShellTool()
        fix.fixFaceTool()
        fix.fixWireTool()
        fix.fixEdgeTool()

        fix.FixSolidMode = True
        self.assertEqual(fix.FixSolidMode, True)

        fix.FixFreeShellMode = True
        self.assertEqual(fix.FixFreeShellMode, True)

        fix.FixFreeFaceMode = True
        self.assertEqual(fix.FixFreeFaceMode, True)

        fix.FixFreeWireMode = True
        self.assertEqual(fix.FixFreeWireMode, True)

        fix.FixSameParameterMode = True
        self.assertEqual(fix.FixSameParameterMode, True)

        fix.FixVertexPositionMode = True
        self.assertEqual(fix.FixVertexPositionMode, True)

        fix.FixVertexTolMode = True
        self.assertEqual(fix.FixVertexTolMode, True)

        fix.perform()

    def testShapeFix_Edge(self):
        surface = Part.Plane()
        face = surface.toShape(-1, 1, -1, 1)

        with self.assertRaises(TypeError):
            Part.ShapeFix.Edge([])

        wirefix = Part.ShapeFix.Wire(face.OuterWire, face, 1e-7)
        fix = wirefix.fixEdgeTool()
        print(fix)

        fix.fixRemovePCurve(face.Edge1, face)
        fix.fixRemovePCurve(face.Edge1, face.Surface, face.Placement)
        with self.assertRaises(TypeError):
            fix.fixRemovePCurve(face)

        fix.fixRemoveCurve3d(face.Edge1)
        fix.fixAddCurve3d(face.Edge1)

        fix.fixAddPCurve(face.Edge1, face, False)
        fix.fixAddPCurve(face.Edge1, face.Surface, face.Placement, False)
        with self.assertRaises(TypeError):
            fix.fixAddPCurve(face)

        fix.fixVertexTolerance(face.Edge1)
        fix.fixVertexTolerance(face.Edge1, face)

        fix.fixReversed2d(face.Edge1, face)
        fix.fixReversed2d(face.Edge1, face.Surface, face.Placement)
        with self.assertRaises(TypeError):
            fix.fixReversed2d(face)

        fix.fixSameParameter(face.Edge1)
        fix.fixSameParameter(face.Edge1, face)
        with self.assertRaises(TypeError):
            fix.fixSameParameter(face)

    def testShapeFix_Face(self):
        surface = Part.Plane()
        face = surface.toShape(-1, 1, -1, 1)

        Part.ShapeFix.Face()
        Part.ShapeFix.Face(surface, 0.00001, True)
        with self.assertRaises(TypeError):
            Part.ShapeFix.Face([])

        fix = Part.ShapeFix.Face(face)
        print(fix)

        fix.fixOrientation()
        fix.fixAddNaturalBound()
        fix.fixMissingSeam()
        fix.fixSmallAreaWire(True)
        fix.fixLoopWire()
        fix.fixIntersectingWires()
        fix.fixWiresTwoCoincidentEdges()
        fix.fixPeriodicDegenerated()
        fix.perform()

        fix.add(face.OuterWire)
        current = fix.face()
        result = fix.result()
        fix.fixWireTool()

        fix.FixWireMode = True
        self.assertEqual(fix.FixWireMode, True)

        fix.FixOrientationMode = True
        self.assertEqual(fix.FixOrientationMode, True)

        fix.FixAddNaturalBoundMode = True
        self.assertEqual(fix.FixAddNaturalBoundMode, True)

        fix.FixMissingSeamMode = True
        self.assertEqual(fix.FixMissingSeamMode, True)

        fix.FixSmallAreaWireMode = True
        self.assertEqual(fix.FixSmallAreaWireMode, True)

        fix.RemoveSmallAreaFaceMode = True
        self.assertEqual(fix.RemoveSmallAreaFaceMode, True)

        fix.FixIntersectingWiresMode = True
        self.assertEqual(fix.FixIntersectingWiresMode, True)

        fix.FixLoopWiresMode = True
        self.assertEqual(fix.FixLoopWiresMode, True)

        fix.FixSplitFaceMode = True
        self.assertEqual(fix.FixSplitFaceMode, True)

        fix.AutoCorrectPrecisionMode = True
        self.assertEqual(fix.AutoCorrectPrecisionMode, True)

        fix.FixPeriodicDegeneratedMode = True
        self.assertEqual(fix.FixPeriodicDegeneratedMode, True)

        fix.clearModes()

    def testShapeFix_Shell(self):
        surface = Part.Plane()
        face = surface.toShape(-1, 1, -1, 1)
        shell = Part.Shell([face])

        Part.ShapeFix.Shell()
        with self.assertRaises(TypeError):
            Part.ShapeFix.Face([])

        fix = Part.ShapeFix.Shell(shell)
        fix.init(shell)
        print(fix)
        fix.perform()
        fix.shell()
        fix.shape()
        fix.fixFaceTool()

        fix.setNonManifoldFlag(True)
        fix.fixFaceOrientation(shell)

        self.assertEqual(len(fix.errorFaces().Faces), 0)

        self.assertEqual(fix.numberOfShells(), 1)

        fix.FixFaceMode = True
        self.assertEqual(fix.FixFaceMode, True)

        fix.FixOrientationMode = True
        self.assertEqual(fix.FixOrientationMode, True)

    def testShapeFix_Solid(self):
        box = Part.makeBox(1, 1, 1)
        with self.assertRaises(TypeError):
            Part.ShapeFix.Solid([])

        fix = Part.ShapeFix.Solid()
        fix.init(box)
        print(fix)

        fix.perform()
        fix.solid()
        fix.shape()
        fix.fixShellTool()
        fix.solidFromShell(box.Shells[0])

        fix.FixShellMode = True
        self.assertEqual(fix.FixShellMode, True)

        fix.FixShellOrientationMode = True
        self.assertEqual(fix.FixShellOrientationMode, True)

        fix.CreateOpenSolidMode = True
        self.assertEqual(fix.CreateOpenSolidMode, True)

    def testShapeFix_Wire(self):
        with self.assertRaises(TypeError):
            Part.ShapeFix.Wire([])

        surface = Part.Plane()
        face = surface.toShape(-1, 1, -1, 1)
        Part.ShapeFix.Wire(face.OuterWire, face, 1e-7)
        fix = Part.ShapeFix.Wire()
        fix.init(face.OuterWire, face, 1e-7)
        fix.load(face.OuterWire)
        fix.setSurface(surface)
        fix.setSurface(surface, face.Placement)
        fix.setFace(face)
        fix.setMaxTailAngle(math.pi)
        fix.setMaxTailWidth(10.0)
        fix.fixEdgeTool()

        self.assertEqual(fix.isLoaded(), True)
        self.assertEqual(fix.isReady(), True)
        self.assertEqual(fix.numberOfEdges(), 4)

        print(fix)
        fix.clearModes()
        fix.clearStatuses()

        fix.wire()
        fix.wireAPIMake()
        fix.face()

        fix.ModifyTopologyMode = True
        self.assertEqual(fix.ModifyTopologyMode, True)

        fix.ModifyGeometryMode = True
        self.assertEqual(fix.ModifyGeometryMode, True)

        fix.ModifyRemoveLoopMode = True
        self.assertEqual(fix.ModifyRemoveLoopMode, True)

        fix.ClosedWireMode = True
        self.assertEqual(fix.ClosedWireMode, True)

        fix.PreferencePCurveMode = True
        self.assertEqual(fix.PreferencePCurveMode, True)

        fix.FixGapsByRangesMode = True
        self.assertEqual(fix.FixGapsByRangesMode, True)

        fix.FixReorderMode = True
        self.assertEqual(fix.FixReorderMode, True)

        fix.FixSmallMode = True
        self.assertEqual(fix.FixSmallMode, True)

        fix.FixConnectedMode = True
        self.assertEqual(fix.FixConnectedMode, True)

        fix.FixEdgeCurvesMode = True
        self.assertEqual(fix.FixEdgeCurvesMode, True)

        fix.FixDegeneratedMode = True
        self.assertEqual(fix.FixDegeneratedMode, True)

        fix.FixSelfIntersectionMode = True
        self.assertEqual(fix.FixSelfIntersectionMode, True)

        fix.FixLackingMode = True
        self.assertEqual(fix.FixLackingMode, True)

        fix.FixGaps3dMode = True
        self.assertEqual(fix.FixGaps3dMode, True)

        fix.FixGaps2dMode = True
        self.assertEqual(fix.FixGaps2dMode, True)

        fix.FixReversed2dMode = True
        self.assertEqual(fix.FixReversed2dMode, True)

        fix.FixRemovePCurveMode = True
        self.assertEqual(fix.FixRemovePCurveMode, True)

        fix.FixAddPCurveMode = True
        self.assertEqual(fix.FixAddPCurveMode, True)

        fix.FixRemoveCurve3dMode = True
        self.assertEqual(fix.FixRemoveCurve3dMode, True)

        fix.FixAddCurve3dMode = True
        self.assertEqual(fix.FixAddCurve3dMode, True)

        fix.FixSeamMode = True
        self.assertEqual(fix.FixSeamMode, True)

        fix.FixShiftedMode = True
        self.assertEqual(fix.FixShiftedMode, True)

        fix.FixSameParameterMode = True
        self.assertEqual(fix.FixSameParameterMode, True)

        fix.FixVertexToleranceMode = True
        self.assertEqual(fix.FixVertexToleranceMode, True)

        fix.FixNotchedEdgesMode = True
        self.assertEqual(fix.FixNotchedEdgesMode, True)

        fix.FixSelfIntersectingEdgeMode = True
        self.assertEqual(fix.FixSelfIntersectingEdgeMode, True)

        fix.FixIntersectingEdgesMode = True
        self.assertEqual(fix.FixIntersectingEdgesMode, True)

        fix.FixNonAdjacentIntersectingEdgesMode = True
        self.assertEqual(fix.FixNonAdjacentIntersectingEdgesMode, True)

        fix.FixTailMode = True
        self.assertEqual(fix.FixTailMode, True)

        fix.perform()
        fix.fixReorder()
        fix.fixSmall(True)
        fix.fixSmall(1, True, 1e-7)
        fix.fixConnected()
        fix.fixConnected(1, True)
        fix.fixEdgeCurves()
        fix.fixDegenerated()
        fix.fixDegenerated(1)
        fix.fixSelfIntersection()
        fix.fixLacking()
        fix.fixLacking(1, False)
        fix.fixClosed()
        fix.fixGaps3d()
        fix.fixGaps2d()
        fix.fixSeam(1)
        fix.fixShifted()
        fix.fixNotchedEdges()
        fix.fixGap3d(1, False)
        fix.fixGap2d(1, False)
        fix.fixTails()


class PartBOPTestContainer(unittest.TestCase):
    def setUp(self):
        self.Doc = FreeCAD.newDocument()

    def testMakeFuse(self):
        box = self.Doc.addObject("Part::Box", "Box")
        cyl = self.Doc.addObject("Part::Cylinder", "Cylinder")
        part = self.Doc.addObject("App::Part", "Part")
        part.addObject(box)
        part.addObject(cyl)
        from BOPTools import BOPFeatures

        bp = BOPFeatures.BOPFeatures(self.Doc)
        fuse = bp.make_multi_fuse([cyl.Name, box.Name])
        self.assertEqual(part, fuse.getParent())

    def testMakeCut(self):
        box = self.Doc.addObject("Part::Box", "Box")
        cyl = self.Doc.addObject("Part::Cylinder", "Cylinder")
        part = self.Doc.addObject("App::Part", "Part")
        part.addObject(box)
        part.addObject(cyl)
        from BOPTools import BOPFeatures

        bp = BOPFeatures.BOPFeatures(self.Doc)
        fuse = bp.make_cut([cyl.Name, box.Name])
        self.assertEqual(part, fuse.getParent())

    def testMakeCommon(self):
        box = self.Doc.addObject("Part::Box", "Box")
        cyl = self.Doc.addObject("Part::Cylinder", "Cylinder")
        part = self.Doc.addObject("App::Part", "Part")
        part.addObject(box)
        part.addObject(cyl)
        from BOPTools import BOPFeatures

        bp = BOPFeatures.BOPFeatures(self.Doc)
        fuse = bp.make_multi_common([cyl.Name, box.Name])
        self.assertEqual(part, fuse.getParent())

    def tearDown(self):
        FreeCAD.closeDocument(self.Doc.Name)


class BSplineCurve2d(unittest.TestCase):
    def setUp(self):
        vec2 = FreeCAD.Base.Vector2d
        self.pts = [vec2(0, 0), vec2(1, 0)]
        self.bs = Part.Geom2d.BSplineCurve2d()

    def testInterpolate(self):
        self.bs.interpolate(Points=self.pts)

    def testApproximate(self):
        self.bs.approximate(Points=self.pts)


class GeometryCurve(unittest.TestCase):
    def testProject(self):
        line = Part.Line()
        line.projectPoint(FreeCAD.Vector())


class EmptyEdge(unittest.TestCase):
    def testParameterByLength(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.getParameterByLength(0)

    def testValueAt(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.valueAt(0)

    def testParameters(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            face = Part.Face()
            edge.parameters(face)

    def testParameterAt(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            vertex = Part.Vertex(0, 0, 0)
            edge.parameterAt(vertex)

    def testTangentAt(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.tangentAt(0)

    def testCurvatureAt(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.curvatureAt(0)

    def testCenterOfCurvatureAt(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.centerOfCurvatureAt(0)

    def testDerivative1At(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.derivative1At(0)

    def testDerivative2At(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.derivative2At(0)

    def testDerivative3At(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.derivative3At(0)

    def testNormalAt(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.normalAt(0)

    def testFirstVertex(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.firstVertex()

    def testLastVertex(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.lastVertex()

    def testGetTolerance(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.Tolerance

    def testSetTolerance(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.Tolerance = 0.01

    def testLength(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.Length

    def testCurve(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.Curve

    def testParameterRange(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.ParameterRange

    def testFirstParameter(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.FirstParameter

    def testLastParameter(self):
        with self.assertRaises(ValueError):
            edge = Part.Edge()
            edge.LastParameter


class EmptyFace(unittest.TestCase):
    def testMakeOffset(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.makeOffset(1)

    def testValueAt(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.valueAt(0, 0)

    def testNormalAt(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.normalAt(0, 0)

    def testTangentAt(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.tangentAt(0, 0)

    def testCurvatureAt(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.curvatureAt(0, 0)

    def testDerivative1At(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.derivative1At(0, 0)

    def testDerivative2At(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.derivative2At(0, 0)

    def testCutHoles(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            circle = Part.Circle()
            wire = Part.Wire(Part.Edge(circle))
            face.cutHoles([wire])

    def testUVNodes(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.getUVNodes()

    def testGetTolerance(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.Tolerance

    def testSetTolerance(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.Tolerance = 0.01

    def testParameterRange(self):
        with self.assertRaises(ValueError):
            face = Part.Face()
            face.ParameterRange


class PartExtrusionTests(unittest.TestCase):

    def setUp(self):
        self.Doc = FreeCAD.newDocument("PartExtrusionTest")

    def _make_face_with_hole(self):
        """Create a 10x10 face with a 6x6 centered hole."""
        outer = Part.makePolygon(
            [
                Base.Vector(-5, -5, 0),
                Base.Vector(5, -5, 0),
                Base.Vector(5, 5, 0),
                Base.Vector(-5, 5, 0),
                Base.Vector(-5, -5, 0),
            ]
        )
        inner = Part.makePolygon(
            [
                Base.Vector(-3, -3, 0),
                Base.Vector(3, -3, 0),
                Base.Vector(3, 3, 0),
                Base.Vector(-3, 3, 0),
                Base.Vector(-3, -3, 0),
            ]
        )
        return Part.makeFace([outer, inner], "Part::FaceMakerBullseye")

    @staticmethod
    def _frustum_volume(height, bottom_area, top_area):
        """Volume of a frustum: h/3 * (A1 + A2 + sqrt(A1*A2))."""
        return height / 3 * (bottom_area + top_area + math.sqrt(bottom_area * top_area))

    def testExtrudeTaperInverted(self):
        """With Inverted taper (Part::Extrusion default), the outer grows
        while the hole shrinks."""
        base_feature = self.Doc.addObject("Part::Feature", "Base")
        base_feature.Shape = self._make_face_with_hole()
        extrusion = self.Doc.addObject("Part::Extrusion", "Extrude")
        extrusion.Base = base_feature
        extrusion.LengthFwd = 10
        extrusion.Solid = True
        extrusion.TaperAngle = 10
        self.Doc.recompute()

        self.assertTrue(extrusion.Shape.isValid())
        offset = 10 * math.tan(math.radians(10))
        outer = self._frustum_volume(10, 10 * 10, (10 + 2 * offset) * (10 + 2 * offset))
        hole = self._frustum_volume(10, 6 * 6, (6 - 2 * offset) * (6 - 2 * offset))
        expected_volume = outer - hole  # ~1204
        self.assertAlmostEqual(extrusion.Shape.Volume, expected_volume, places=1)

    def testExtrudeTaperSameAsOuter(self):
        """Regression test for issue #28709: with SameAsOuter taper (the
        PartDesign path), both wires must taper in the same direction."""
        base_feature = self.Doc.addObject("Part::Feature", "Base")
        base_feature.Shape = self._make_face_with_hole()
        extrusion = self.Doc.addObject("Part::Extrusion", "Extrude")
        extrusion.Base = base_feature
        extrusion.LengthFwd = 10
        extrusion.Solid = True
        extrusion.TaperAngle = 10
        extrusion.InnerWireTaper = "SameAsOuter"
        self.Doc.recompute()

        self.assertTrue(extrusion.Shape.isValid())
        offset = 10 * math.tan(math.radians(10))
        outer = self._frustum_volume(10, 10 * 10, (10 + 2 * offset) * (10 + 2 * offset))
        hole = self._frustum_volume(10, 6 * 6, (6 + 2 * offset) * (6 + 2 * offset))
        expected_volume = outer - hole  # ~781
        self.assertAlmostEqual(extrusion.Shape.Volume, expected_volume, places=1)

    def tearDown(self):
        FreeCAD.closeDocument("PartExtrusionTest")



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2024 Ondsel <development@ondsel.com>                    *
# *   Copyright (c) 2025 Billy Huddleston <billy@ivdc.com>                  *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

from string import Template
import FreeCAD
import Path.Log
import base64
import os

from Path.Main.Sanity.HTMLTemplate import (
    html_template,
    base_template,
    squawk_template,
    tool_template,
    op_run_template,
    op_tool_template,
    tool_item_template,
)

translate = FreeCAD.Qt.translate

if False:
    Path.Log.setLevel(Path.Log.Level.DEBUG, Path.Log.thisModule())
    Path.Log.trackModule(Path.Log.thisModule())
else:
    Path.Log.setLevel(Path.Log.Level.INFO, Path.Log.thisModule())


class ReportGenerator:
    def bytes_to_base64_with_tag(self, image_bytes, mime_type="image/png", alt="Image"):
        """
        Takes image bytes and returns (base64_string, <img> tag) for embedding in HTML.
        Default mime_type is image/png.
        """

        if not image_bytes:
            return "", ""
        encoded_string = base64.b64encode(image_bytes).decode()
        html_tag = f'<img src="data:{mime_type};base64,{encoded_string}" alt="{alt}" />'
        return encoded_string, html_tag

    def __init__(self, data, embed_images=False):
        self.embed_images = embed_images
        self.squawks = ""
        self.tools = ""
        self.run_summary_ops = ""
        self.formatted_data = {}
        self.translated_labels = {
            "dateLabel": translate("CAM_Sanity", "Date"),
            "datumLabel": translate("CAM_Sanity", "Part Datum"),
            "descriptionLabel": translate("CAM_Sanity", "Description"),
            "diameterLabel": translate("CAM_Sanity", "Tool Diameter"),
            "feedLabel": translate("CAM_Sanity", "Feed Rate"),
            "fileSizeLabel": translate("CAM_Sanity", "File Size (kB)"),
            "fixturesLabel": translate("CAM_Sanity", "Fixtures"),
            "flagsLabel": translate("CAM_Sanity", "Post Processor Flags"),
            "gcodeFileLabel": translate("CAM_Sanity", "G-code File"),
            "headingLabel": translate("CAM_Sanity", "Setup Report for CAM Job"),
            "inspectionNotesLabel": translate("CAM_Sanity", "Inspection Notes"),
            "lastpostLabel": translate("CAM_Sanity", "Last Post Process Date"),
            "lineCountLabel": translate("CAM_Sanity", "Line Count"),
            "machineLabel": translate("CAM_Sanity", "Machine"),
            "manufLabel": translate("CAM_Sanity", "Manufacturer"),
            "materialLabel": translate("CAM_Sanity", "Material"),
            "noteLabel": translate("CAM_Sanity", "Note"),
            "offsetsLabel": translate("CAM_Sanity", "Work Offsets"),
            "opLabel": translate("CAM_Sanity", "Operation"),
            "operatorLabel": translate("CAM_Sanity", "Operator"),
            "orderByLabel": translate("CAM_Sanity", "Order By"),
            "outputLabel": translate("CAM_Sanity", "Output (G-code)"),
            "partInformationLabel": translate("CAM_Sanity", "Part Information"),
            "partNumberLabel": translate("CAM_Sanity", "Part Number"),
            "postLabel": translate("CAM_Sanity", "Postprocessor"),
            "programmerLabel": translate("CAM_Sanity", "Programmer"),
            "roughStockLabel": translate("CAM_Sanity", "Rough Stock"),
            "runSummaryLabel": translate("CAM_Sanity", "Run Summary"),
            "shapeLabel": translate("CAM_Sanity", "Tool Shape"),
            "speedLabel": translate("CAM_Sanity", "Spindle Speed"),
            "squawksLabel": translate("CAM_Sanity", "Squawks"),
            "stopsLabel": translate("CAM_Sanity", "Stops"),
            "sSpeedCarbideLabel": translate("CAM_Sanity", "Surface Speed Carbide"),
            "sSpeedHSSLabel": translate("CAM_Sanity", "Surface Speed HSS"),
            "tableOfContentsLabel": translate("CAM_Sanity", "Table of Contents"),
            "tcLabel": translate("CAM_Sanity", "Tool Controller"),
            "toolDataLabel": translate("CAM_Sanity", "Tool Data"),
            "toolNumberLabel": translate("CAM_Sanity", "Tool Number"),
            "urlLabel": translate("CAM_Sanity", "URL"),
            "xDimLabel": translate("CAM_Sanity", "X Size"),
            "yDimLabel": translate("CAM_Sanity", "Y Size"),
            "zDimLabel": translate("CAM_Sanity", "Z Size"),
            "jobMinZLabel": translate("CAM_Sanity", "Minimum Z"),
            "jobMaxZLabel": translate("CAM_Sanity", "Maximum Z"),
            "coolantLabel": translate("CAM_Sanity", "Coolant Mode"),
            "cycleTimeLabel": translate("CAM_Sanity", "Cycle Time"),
            "PartLabel": translate("CAM_Sanity", "Part"),
            "SequenceLabel": translate("CAM_Sanity", "Sequence"),
            "JobTypeLabel": translate("CAM_Sanity", "Job Type"),
            "CADLabel": translate("CAM_Sanity", "CAD File"),
            "LastSaveLabel": translate("CAM_Sanity", "Last Save"),
            "CustomerLabel": translate("CAM_Sanity", "Customer"),
        }

        # format the data for all blocks except tool
        for block in [
            "baseData",
            "designData",
            "runData",
            "outputData",
            "fixtureData",
            "stockData",
        ]:
            for key, val in data[block].items():
                Path.Log.debug(f"key: {key} val: {val}")
                if key == "squawkData":
                    self._format_squawks(val)
                elif key == "bases":
                    self._format_bases(val)
                elif key == "operations":
                    self._format_run_summary_ops(val)
                elif key in ["baseimage", "imagepath", "datumImage", "stockImage"]:
                    Path.Log.debug(f"key: {key} val: {val}")
                    if self.embed_images:
                        Path.Log.debug("Embedding images")
                        if isinstance(val, bytes):
                            encoded_image, tag = self.bytes_to_base64_with_tag(
                                val, mime_type="image/png", alt=key
                            )
                        else:
                            encoded_image, tag = self.file_to_base64_with_tag(val)
                    else:
                        Path.Log.debug("Not Embedding images")
                        tag = f"<img src={val} name='Image' alt={key} />"
                    self.formatted_data[key] = tag
                else:
                    self.formatted_data[key] = val

        # format the data for the tool block
        for key, val in data["toolData"].items():
            if key == "squawkData":
                self._format_squawks(val)
            else:
                toolNumber = key
                toolAttributes = val
                # Prefer imagebytes for embedding if present
                if (
                    self.embed_images
                    and "imagebytes" in toolAttributes
                    and toolAttributes["imagebytes"]
                ):
                    _, tag = self.bytes_to_base64_with_tag(
                        toolAttributes["imagebytes"], mime_type="image/png", alt=key
                    )
                    toolAttributes["imagepath"] = tag
                elif "imagepath" in toolAttributes and toolAttributes["imagepath"] != "":
                    if self.embed_images:
                        encoded_image, tag = self.file_to_base64_with_tag(
                            toolAttributes["imagepath"]
                        )
                    else:
                        tag = f"<img src={toolAttributes['imagepath']} name='Image' alt={key} />"
                    toolAttributes["imagepath"] = tag

                self._format_tool(key, val)

        self.formatted_data["squawks"] = self.squawks
        self.formatted_data["run_summary_ops"] = self.run_summary_ops
        self.formatted_data["tool_data"] = self.tools
        self.formatted_data["tool_list"] = self._format_tool_list(data["toolData"])

        # Path.Log.debug(self.formatted_data)

    def _format_tool_list(self, tool_data):
        tool_list = ""
        for key, val in tool_data.items():
            if key == "squawkData":
                continue
            else:
                val["toolNumber"] = key
                tool_list += tool_item_template.substitute(val)
        return tool_list

    def _format_run_summary_ops(self, op_data):
        for op in op_data:
            self.run_summary_ops += op_run_template.substitute(op)

    def _format_tool(self, tool_number, tool_data):
        td = {}
        td["toolNumber"] = tool_number

        for key, val in tool_data.items():
            if key == "squawkData":
                self._format_squawks(val)
            if key == "ops":
                opslist = ""
                for op in val:
                    op.update(self.translated_labels)
                    opslist += op_tool_template.substitute(op)
                td[key] = opslist
            else:
                td[key] = val

        td.update(self.translated_labels)
        Path.Log.debug(f"Tool data: {td}")

        self.tools += tool_template.substitute(td)

    def _format_bases(self, base_data):
        bases = ""
        for key, val in base_data.items():
            bases += base_template.substitute({"key": key, "val": val})
        self.formatted_data["bases"] = bases

    def _format_squawks(self, squawk_data):
        for squawk in squawk_data:
            if self.embed_images:
                data, tag = self.file_to_base64_with_tag(squawk["squawkIcon"])
            else:
                tag = f"<img src={squawk['squawkIcon']} name='Image' alt='TIP' />"

            squawk["squawkIcon"] = tag
            self.squawks += squawk_template.substitute(squawk)

    def generate_html(self):
        self.formatted_data.update(self.translated_labels)
        html_content = html_template.substitute(self.formatted_data)
        return html_content

    def encode_gcode_to_base64(filepath):
        with open(filepath, "rb") as file:
            encoded_string = base64.b64encode(file.read()).decode("utf-8")
        return encoded_string

    def file_to_base64_with_tag(self, file_path):
        # Determine MIME type based on the file extension
        mime_types = {
            ".jpg": "image/jpeg",
            ".jpeg": "image/jpeg",
            ".png": "image/png",
            ".svg": "image/svg+xml",
            ".gcode": "application/octet-stream",  # MIME type for G-code files
        }
        extension = os.path.splitext(file_path)[1]
        mime_type = mime_types.get(
            extension, "application/octet-stream"
        )  # Default to binary data type if unknown

        if not os.path.exists(file_path):
            Path.Log.error(f"File not found: {file_path}")
            return "", ""

        try:
            # Encode file to base64
            with open(file_path, "rb") as file:
                encoded_string = base64.b64encode(file.read()).decode()

            # Generate HTML tag based on file type
            if extension in [".jpg", ".jpeg", ".png", ".svg"]:
                html_tag = f'<img src="data:{mime_type};base64,{encoded_string}">'
            elif extension in [".gcode", ".nc", ".tap", ".cnc"]:
                html_tag = f'<a href="data:{mime_type};base64,{encoded_string}" download="{os.path.basename(file_path)}">Download G-code File</a>'
            else:
                html_tag = f'<a href="data:{mime_type};base64,{encoded_string}" download="{os.path.basename(file_path)}">Download File</a>'

            return encoded_string, html_tag
        except FileNotFoundError:
            Path.Log.error(f"File not found: {file_path}")
            return "", ""



from Base.Metadata import export
from Spreadsheet.Sheet import Sheet

@export(
    Include="Mod/Assembly/App/BomObject.h",
    FatherInclude="Mod/Spreadsheet/App/SheetPy.h",
    Namespace="Assembly",
)
class BomObject(Sheet):
    """
    This class is the BOM object of assemblies, it derives from Spreadsheet::Sheet.

    Author: Ondsel (development@ondsel.com)
    License: LGPL-2.1-or-later
    """



# SPDX-License-Identifier: LGPL-2.1-or-later
# /**************************************************************************
#                                                                           *
#    Copyright (c) 2023 Ondsel <development@ondsel.com>                     *
#                                                                           *
#    This file is part of FreeCAD.                                          *
#                                                                           *
#    FreeCAD is free software: you can redistribute it and/or modify it     *
#    under the terms of the GNU Lesser General Public License as            *
#    published by the Free Software Foundation, either version 2.1 of the   *
#    License, or (at your option) any later version.                        *
#                                                                           *
#    FreeCAD is distributed in the hope that it will be useful, but         *
#    WITHOUT ANY WARRANTY; without even the implied warranty of             *
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
#    Lesser General Public License for more details.                        *
#                                                                           *
#    You should have received a copy of the GNU Lesser General Public       *
#    License along with FreeCAD. If not, see                                *
#    <https://www.gnu.org/licenses/>.                                       *
#                                                                           *
# **************************************************************************/

import os
import FreeCAD as App

from PySide.QtCore import QT_TRANSLATE_NOOP

if App.GuiUp:
    import FreeCADGui as Gui
    from PySide import QtCore, QtGui, QtWidgets

import JointObject
from JointObject import TaskAssemblyCreateJoint
import UtilsAssembly
import Assembly_rc

# translate = App.Qt.translate

__title__ = "Assembly Commands to Create Joints"
__author__ = "Ondsel"
__url__ = "https://www.freecad.org"


def noOtherTaskActive():
    return UtilsAssembly.isAssemblyCommandActive() or JointObject.activeTask is not None


def isCreateJointActive():
    return UtilsAssembly.assembly_has_at_least_n_parts(1) and noOtherTaskActive()


def activateJoint(index):
    if JointObject.activeTask:
        JointObject.activeTask.reject()

    Gui.addModule("JointObject")  # NOLINT
    Gui.doCommand(f"panel = JointObject.TaskAssemblyCreateJoint({index})")
    Gui.doCommandGui("dialog = Gui.Control.showDialog(panel)")
    dialog = Gui.doCommandEval("dialog")
    if dialog is not None:
        dialog.setAutoCloseOnTransactionChange(True)
        dialog.setAutoCloseOnDeletedDocument(True)
        dialog.setDocumentName(App.ActiveDocument.Name)


class CommandCreateJointFixed:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointFixed",
            "MenuText": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointFixed",
                "Fixed Joint",
            ),
            "Accel": "F",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointFixed",
                "<p>1 - If an assembly is active : Creates a joint permanently locking two parts together, preventing any movement or rotation</p>"
                "<p>2 - If a part is active: Positions sub-parts by matching selected coordinate systems. The second part selected will move.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        if UtilsAssembly.activePart() is not None:
            return UtilsAssembly.assembly_has_at_least_n_parts(2)

        return isCreateJointActive()

    def Activated(self):
        activateJoint(0)


class CommandCreateJointRevolute:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointRevolute",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointRevolute", "Revolute Joint"),
            "Accel": "R",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointRevolute",
                "Creates a revolute joint allowing rotation around a single axis between selected parts",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(1)


class CommandCreateJointCylindrical:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointCylindrical",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointCylindrical", "Cylindrical Joint"),
            "Accel": "C",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointCylindrical",
                "Creates a cylindrical joint that allows rotation around and translation along a single axis between assembled parts",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(2)


class CommandCreateJointSlider:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointSlider",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointSlider", "Slider Joint"),
            "Accel": "S",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointSlider",
                "Creates a slider joint that allows linear movement along a single axis, but restricts rotation between selected parts",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(3)


class CommandCreateJointBall:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointBall",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointBall", "Ball Joint"),
            "Accel": "B",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointBall",
                "Creates a ball joint that connects parts at a point, allowing unrestricted movement as long as the connection points remain in contact",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(4)


class CommandCreateJointDistance:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointDistance",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointDistance", "Distance Joint"),
            "Accel": "D",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointDistance",
                "<p>Creates a distance joint that fixes the distance between the selected objects</p>"
                "<p>Creates one of several different joints based on the selection. "
                "For example, a distance of 0 between a plane and a cylinder creates a tangent joint. A distance of 0 between planes will make them co-planar.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(5)


class CommandCreateJointParallel:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointParallel",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointParallel", "Parallel Joint"),
            "Accel": "N",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointParallel",
                "Creates a parallel joint that makes the Z-axis of the selected coordinate systems parallel",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(6)


class CommandCreateJointPerpendicular:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointPerpendicular",
            "MenuText": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointPerpendicular", "Perpendicular Joint"
            ),
            "Accel": "M",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointPerpendicular",
                "Creates a perpendicular joint that makes the Z-axis of the selected coordinate systems perpendicular",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(7)


class CommandCreateJointAngle:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointAngle",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointAngle", "Angle Joint"),
            "Accel": "X",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointAngle",
                "Creates an angle joint that fixes the angle between the Z-axis of the selected coordinate systems",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(8)


class CommandCreateJointRackPinion:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointRackPinion",
            "MenuText": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointRackPinion", "Rack and Pinion Joint"
            ),
            "Accel": "Q",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointRackPinion",
                "<p>Creates a rack and pinion joint that links a part with a sliding joint to a part with a revolute joint</p>"
                "<p>Selects the same coordinate systems as the revolute and sliding joints. The pitch radius defines the movement ratio between the rack and the pinion.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(9)


class CommandCreateJointScrew:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointScrew",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointScrew", "Screw Joint"),
            "Accel": "W",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointScrew",
                "<p>Creates a screw joint that links a part with a sliding joint to a part with a revolute joint</p>"
                "<p>Select the same coordinate systems as the revolute and sliding joints. The pitch radius defines the movement ratio between the rotating screw and the sliding part.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(10)


class CommandCreateJointGears:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointGears",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointGears", "Gears Joint"),
            "Accel": "T",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointGears",
                "<p>Creates a gears joint that links 2 rotating gears together. They will have inverse rotation direction.</p>"
                "<p>Select the same coordinate systems as the revolute joints.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(11)


class CommandCreateJointBelt:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_CreateJointPulleys",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointBelt", "Belt Joint"),
            "Accel": "L",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointBelt",
                "<p>Creates a belt joint that links 2 rotating objects together. They will have the same rotation direction.</p>"
                "<p>Select the same coordinate systems as the revolute joints.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()

    def Activated(self):
        activateJoint(12)


class CommandGroupGearBelt:
    def GetCommands(self):
        return ("Assembly_CreateJointGears", "Assembly_CreateJointBelt")

    def GetResources(self):
        """Set icon, menu and tooltip."""
        return {
            "Pixmap": "Assembly_CreateJointGears",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_CreateJointGearBelt", "Gears/Belt Joint"),
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_CreateJointGearBelt",
                "<p>Creates a gears or belt joint that links 2 rotating gears together</p>"
                "<p>Select the same coordinate systems as the revolute joints.</p>",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return isCreateJointActive()


def createGroundedJoint(obj):
    if not UtilsAssembly.activeAssembly():
        return

    Gui.addModule("UtilsAssembly")
    Gui.addModule("JointObject")
    commands = (
        f'obj = App.ActiveDocument.getObject("{obj.Name}")\n'
        "assembly = UtilsAssembly.activeAssembly()\n"
        "joint_group = UtilsAssembly.getJointGroup(assembly)\n"
        'ground = joint_group.newObject("App::FeaturePython", "GroundedJoint")\n'
        "JointObject.GroundedJoint(ground, obj)"
    )
    Gui.doCommand(commands)
    Gui.doCommandGui("JointObject.ViewProviderGroundedJoint(ground.ViewObject)")

    Gui.doCommand("UtilsAssembly.activeAssembly().Document.recompute()")
    return Gui.doCommandEval("ground")


class CommandToggleGrounded:
    def __init__(self):
        pass

    def GetResources(self):
        return {
            "Pixmap": "Assembly_ToggleGrounded",
            "MenuText": QT_TRANSLATE_NOOP("Assembly_ToggleGrounded", "Toggle Grounded"),
            "Accel": "G",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Assembly_ToggleGrounded",
                "<p>Toggles the grounding of a part.</p>"
                "<p>Grounding a part permanently locks its position in the assembly, preventing any movement or rotation. You need at least one grounded part before starting to assemble.",
            ),
            "CmdType": "ForEdit",
        }

    def IsActive(self):
        return (
            UtilsAssembly.isAssemblyCommandActive()
            and UtilsAssembly.assembly_has_at_least_n_parts(1)
        )

    def Activated(self):
        assembly = UtilsAssembly.activeAssembly()
        if not assembly:
            return

        joint_group = UtilsAssembly.getJointGroup(assembly)

        selection = Gui.Selection.getSelectionEx("*", 0)
        if not selection:
            return

        App.setActiveTransaction("Toggle grounded")
        for sel in selection:
            # If you select 2 solids (bodies for example) within an assembly.
            # There'll be a single sel but 2 SubElementNames.
            for sub in sel.SubElementNames:
                # First check if selection is a grounded object
                resolved = sel.Object.resolveSubElement(sub)
                if resolved:
                    obj = resolved[0]
                    if hasattr(obj, "ObjectToGround"):
                        commands = (
                            "doc = App.ActiveDocument\n"
                            f'doc.removeObject("{obj.Name}")\n'
                            "doc.recompute()\n"
                        )
                        Gui.doCommand(commands)
                        continue

                moving_part, new_sub = UtilsAssembly.getComponentReference(
                    assembly, sel.Object, sub
                )
                if not moving_part:
                    continue

                # Only objects within the assembly.
                if moving_part is None:
                    continue

                # Check if part is grounded and if so delete the joint.
                ungrounded = False
                for joint in joint_group.Group:
                    if hasattr(joint, "ObjectToGround") and joint.ObjectToGround == moving_part:
                        commands = (
                            "doc = App.ActiveDocument\n"
                            f'doc.removeObject("{joint.Name}")\n'
                            "doc.recompute()\n"
                        )
                        Gui.doCommand(commands)
                        ungrounded = True
                        break
                if ungrounded:
                    continue

                # Create groundedJoint.
                createGroundedJoint(moving_part)
        App.closeActiveTransaction()


if App.GuiUp:
    Gui.addCommand("Assembly_ToggleGrounded", CommandToggleGrounded())
    Gui.addCommand("Assembly_CreateJointFixed", CommandCreateJointFixed())
    Gui.addCommand("Assembly_CreateJointRevolute", CommandCreateJointRevolute())
    Gui.addCommand("Assembly_CreateJointCylindrical", CommandCreateJointCylindrical())
    Gui.addCommand("Assembly_CreateJointSlider", CommandCreateJointSlider())
    Gui.addCommand("Assembly_CreateJointBall", CommandCreateJointBall())
    Gui.addCommand("Assembly_CreateJointDistance", CommandCreateJointDistance())
    Gui.addCommand("Assembly_CreateJointParallel", CommandCreateJointParallel())
    Gui.addCommand("Assembly_CreateJointPerpendicular", CommandCreateJointPerpendicular())
    Gui.addCommand("Assembly_CreateJointAngle", CommandCreateJointAngle())
    Gui.addCommand("Assembly_CreateJointRackPinion", CommandCreateJointRackPinion())
    Gui.addCommand("Assembly_CreateJointScrew", CommandCreateJointScrew())
    Gui.addCommand("Assembly_CreateJointGears", CommandCreateJointGears())
    Gui.addCommand("Assembly_CreateJointBelt", CommandCreateJointBelt())
    Gui.addCommand("Assembly_CreateJointGearBelt", CommandGroupGearBelt())



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2024 FreeCAD Team                                       *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENSE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the FreeCAD         *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""Unit tests for the ToolBitBrowserWidget."""

import unittest
from unittest.mock import MagicMock
from typing import cast
from Path.Tool.toolbit.ui.browser import ToolBitBrowserWidget, ToolBitUriRole
from Path.Tool.toolbit.ui.tablecell import TwoLineTableCell
from Path.Tool.toolbit.models.base import ToolBit
from .PathTestUtils import PathTestWithAssets


class TestToolBitBrowserWidget(PathTestWithAssets):
    """Tests for ToolBitBrowserWidget using real assets and widgets."""

    def setUp(self):
        super().setUp()  # Call the base class setUp to initialize assets
        # The browser widget uses the global cam_assets, which is set up
        # by PathTestWithAssets.
        self.widget = ToolBitBrowserWidget(self.assets)

    def test_initial_fetch(self):
        # Simulate expose to trigger initial fetch
        self.widget.showEvent(None)

        # Verify that the list widget is populated after initialization
        # The default test assets include some toolbits.
        self.assertGreater(self.widget._tool_list_widget.count(), 0)

        # Verify apply_filter was called on the list widget with empty string
        # We can check the search_highlight property on the cell widgets
        # as apply_filter sets this.
        for i in range(self.widget._tool_list_widget.count()):
            item = self.widget._tool_list_widget.item(i)
            cell = self.widget._tool_list_widget.itemWidget(item)
            self.assertIsInstance(cell, TwoLineTableCell)
            self.assertEqual(cell.search_highlight, "")

    def test_search_filtering(self):
        # Simulate typing in the search box
        search_term = "Endmill"
        self.widget._search_edit.setText(search_term)

        # Directly trigger the filtering logic
        self.widget._update_list()

        # Verify that the filter was applied to the list widget
        # We can check if items are hidden/shown based on the filter term
        # This requires knowing the content of the test assets.
        # Assuming '5mm_Endmill' and '10mm_Endmill' contain "Endmill"
        # and 'BallEndmill_3mm' does not.

        # Re-fetch assets to know their labels/names for verification
        all_assets = self.assets.fetch(asset_type="toolbit", depth=0)
        expected_visible_uris = set()
        for asset in all_assets:
            tb = cast(ToolBit, asset)
            is_expected = (
                search_term.lower() in tb.label.lower() or search_term.lower() in tb.summary.lower()
            )
            if is_expected:
                expected_visible_uris.add(str(tb.get_uri()))

        # Simulate expose to trigger initial fetch
        self.widget.showEvent(None)

        actual_visible_uris = set()
        for i in range(self.widget._tool_list_widget.count()):
            item = self.widget._tool_list_widget.item(i)
            cell = self.widget._tool_list_widget.itemWidget(item)
            self.assertIsInstance(cell, TwoLineTableCell)
            item_uri = item.data(ToolBitUriRole)

            # Verify highlight was called on all cells
            self.assertEqual(cell.search_highlight, search_term)

            if not item.isHidden():
                actual_visible_uris.add(item_uri)

        self.assertEqual(actual_visible_uris, expected_visible_uris)

    def test_tool_selected_signal(self):
        mock_slot = MagicMock()
        self.widget.toolSelected.connect(mock_slot)

        # Select the first item in the list widget
        if self.widget._tool_list_widget.count() == 0:
            self.skipTest("Not enough toolbits for selection test.")
        first_item = self.widget._tool_list_widget.item(0)
        self.widget._tool_list_widget.setCurrentItem(first_item)

        # Verify signal was emitted with the correct URI
        expected_uri = first_item.data(ToolBitUriRole)
        mock_slot.assert_called_once_with(expected_uri)

    def test_tool_edit_requested_signal(self):
        mock_slot = MagicMock()
        self.widget.itemDoubleClicked.connect(mock_slot)

        # Double-click the first item in the list widget
        if self.widget._tool_list_widget.count() == 0:
            self.skipTest("Not enough toolbits for double-click test.")

        first_item = self.widget._tool_list_widget.item(0)
        # Simulate double-click signal emission from the list widget
        self.widget._tool_list_widget.itemDoubleClicked.emit(first_item)

        # Verify signal was emitted with the correct URI
        expected_uri = first_item.data(ToolBitUriRole)
        mock_slot.assert_called_once_with(expected_uri)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: LGPL-2.1-or-later

from Metadata import constmethod
from BaseClass import BaseClass
from typing import Final

class Persistence(BaseClass):
    """
    Base.Persistence class.

    Class to dump and restore the content of an object.

    Author: Juergen Riegel (FreeCAD@juergen-riegel.net)
    Licence: LGPL
    """

    Content: Final[str] = ""
    """Content of the object in XML representation."""

    MemSize: Final[int] = 0
    """Memory size of the object in bytes."""

    @constmethod
    def dumpContent(self, *, Compression: int = 3) -> bytearray:
        """
        dumpContent(Compression=3) -> bytearray

        Dumps the content of the object, both the XML representation and the additional
        data files required, into a byte representation.

        Compression : int
            Set the data compression level in the range [0,9]. Set to 0 for no compression.
        """
        ...

    def restoreContent(self, obj: object) -> None:
        # TODO: Starting with Python 3.12, collections.abc.Buffer can be used for type hinting
        """
        restoreContent(obj) -> None

        Restore the content of the object from a byte representation as stored by `dumpContent`.
        It could be restored from any Python object implementing the buffer protocol.

        obj : buffer
            Object with buffer protocol support.
        """
        ...



# ***************************************************************************
# *   Copyright (c) 2017 Markus Hovorka <m.hovorka@live.de>                 *
# *   Copyright (c) 2020 Bernd Hahnebach <bernd@bimstatik.org>              *
# *   Copyright (c) 2024 Mario Passaglia <mpassaglia@cbc.uba.ar>            *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD FEM constraint body heat source document object"
__author__ = "Markus Hovorka, Bernd Hahnebach, Mario Passaglia"
__url__ = "https://www.freecad.org"

## @package constraint_bodyheatsource
#  \ingroup FEM
#  \brief constraint body heat source object

import FreeCAD

from . import base_fempythonobject

_PropHelper = base_fempythonobject._PropHelper


class ConstraintBodyHeatSource(base_fempythonobject.BaseFemPythonObject):

    Type = "Fem::ConstraintBodyHeatSource"

    def __init__(self, obj):
        super().__init__(obj)

        for prop in self._get_properties():
            prop.add_to_object(obj)

    def _get_properties(self):
        prop = []

        prop.append(
            _PropHelper(
                type="App::PropertyDissipationRate",
                name="DissipationRate",
                group="Constraint Body Heat Source",
                doc="Power dissipated per unit mass",
                value="0 W/kg",
            )
        )
        prop.append(
            _PropHelper(
                type="App::PropertyPower",
                name="TotalPower",
                group="Constraint Body Heat Source",
                doc="Total power dissipated",
                value="0 W",
            )
        )
        prop.append(
            _PropHelper(
                type="App::PropertyEnumeration",
                name="Mode",
                group="Constraint Body Heat Source",
                doc="Switch quantity input mode",
                value=["Dissipation Rate", "Total Power"],
            )
        )
        prop.append(
            _PropHelper(
                type="App::PropertyBool",
                name="EnableAmplitude",
                group="Constraint Body Heat Source",
                doc="Amplitude of the body heat source",
                value=False,
            )
        )
        prop.append(
            _PropHelper(
                type="App::PropertyStringList",
                name="AmplitudeValues",
                group="Constraint Body Heat Source",
                doc="Amplitude values",
                value=["0, 0", "1, 1"],
            )
        )
        return prop

    def onDocumentRestored(self, obj):
        # update old project with new properties
        for prop in self._get_properties():
            try:
                obj.getPropertyByName(prop.name)
            except FreeCAD.Base.PropertyError:
                prop.add_to_object(obj)

        # migrate old HeatSource property
        try:
            value = obj.getPropertyByName("HeatSource")
            obj.DissipationRate = FreeCAD.Units.Quantity(value, "W/kg")
            obj.Mode = "Dissipation Rate"
            obj.setPropertyStatus("HeatSource", "-LockDynamic")
            obj.removeProperty("HeatSource")
        except FreeCAD.Base.PropertyError:
            pass



from Base.Metadata import export, constmethod
from Part.App.GeometryExtension import GeometryExtension
from typing import Final, overload

@export(
    PythonName="Sketcher.ExternalGeometryExtension",
    Include="Mod/Sketcher/App/ExternalGeometryExtension.h",
    FatherInclude="Mod/Part/App/GeometryExtensionPy.h",
    Constructor=True,
)
class ExternalGeometryExtension(GeometryExtension):
    """
    Describes a ExternalGeometryExtension

    Author: Abdullah Tahiri (abdullah.tahiri.yo@gmail.com)
    Licence: LGPL
    """

    @constmethod
    def testFlag(self) -> bool:
        """
        Returns a boolean indicating whether the given bit is set.
        """
        ...

    def setFlag(self) -> None:
        """
        Sets the given bit to true/false.
        """
        ...
    Ref: str = ""
    """Returns the reference string of this external geometry."""



from Base.Metadata import export, constmethod
from Base.PyObjectBase import PyObjectBase
from Part.HLRBRep_PolyAlgo import HLRBRep_PolyAlgo
from Part.TopoShapePy import TopoShape
from typing import Optional, overload

@export(
    PythonName="Part.PolyHLRToShapePy",
    Twin="HLRBRep_PolyHLRToShape",
    TwinPointer="HLRBRep_PolyHLRToShape",
    Include="HLRBRep_PolyHLRToShape.hxx",
    Constructor=True,
    Delete=True,
)
class PolyHLRToShape(PyObjectBase):
    """
    PolyHLRToShape(algo: HLRBRep_PolyAlgo) -> HLRBRep_PolyHLRToShape

    A framework for filtering the computation results of an HLRBRep_PolyAlgo
    algorithm by extraction.  From the results calculated by the algorithm on a
    shape, a filter returns the type of edge you want to identify.  You can choose
    any of the following types of output:
    - visible sharp edges
    - hidden sharp edges
    - visible smooth edges
    - hidden smooth edges
    - visible sewn edges
    - hidden sewn edges
    - visible outline edges
    - hidden outline edges
    - visible isoparameters and
    - hidden isoparameters.

    Sharp edges present a C0 continuity (non G1). Smooth edges present a G1
    continuity (non G2). Sewn edges present a C2 continuity. The result is composed
    of 2D edges in the projection plane of the view which the algorithm has worked
    with. These 2D edges are not included in the data structure of the visualized
    shape. In order to obtain a complete image, you must combine the shapes given
    by each of the chosen filters. The construction of the shape does not call a
    new computation of the algorithm, but only reads its internal results.
    """

    def update(self, algo: HLRBRep_PolyAlgo) -> None:
        """
        update(algo: HLRBRep_PolyAlgo)
        """
        ...

    def show(self) -> None:
        """
        show()
        """
        ...

    def hide(self) -> None:
        """
        hide()
        """
        ...

    def vCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        vCompound(Shape=None) -> TopoShape

        Sets the extraction filter for visible sharp edges for either shape Shape or
        for all added shapes (Shape=None).
        """
        ...

    def Rg1LineVCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        Rg1LineVCompound(Shape=None) -> TopoShape

        Sets the extraction filter for visible smooth edges for either shape Shape or
        for all added shapes (Shape=None).
        """
        ...

    def RgNLineVCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        RgNLineVCompound(Shape=None) -> TopoShape

        Sets the extraction filter for visible sewn edges for either shape Shape or for
        all added shapes (Shape=None).
        """
        ...

    def outLineVCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        outLineVCompound(Shape=None) -> TopoShape

        Sets the extraction filter for visible outline edges for either shape Shape or
        for all added shapes (Shape=None).
        """
        ...

    def hCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        hCompound(Shape=None) -> TopoShape

        Sets the extraction filter for hidden sharp edges for either shape Shape or for
        all added shapes (Shape=None).
        """
        ...

    def Rg1LineHCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        Rg1LineHCompound(Shape=None) -> TopoShape

        Sets the extraction filter for hidden smooth edges for either shape Shape or
        for all added shapes (Shape=None).
        """
        ...

    def RgNLineHCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        RgNLineHCompound(Shape=None) -> TopoShape

        Sets the extraction filter for hidden sewn edges for either shape Shape or for
        all added shapes (Shape=None).
        """
        ...

    def outLineHCompound(self, Shape: Optional[TopoShape] = None) -> TopoShape:
        """
        outLineHCompound(Shape=None) -> TopoShape

        Sets the extraction filter for hidden outline edges for either shape Shape or
        for all added shapes (Shape=None).
        """
        ...



# ***************************************************************************
# *   Copyright (c) 2025 Stefan Tröger <stefantroeger@gmx.net>              *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD post line plot"
__author__ = "Stefan Tröger"
__url__ = "https://www.freecad.org"

## @package post_lineplot
#  \ingroup FEM
#  \brief Post processing plot displaying lines

# check vtk version to potentially find missmatchs
from femguiutils.vtk_module_handling import vtk_module_handling

vtk_module_handling()

from . import base_fempostextractors
from . import base_fempostvisualizations
from . import post_extract2D

from femguiutils import post_visualization

# register visualization and extractors
post_visualization.register_visualization(
    "Lineplot", ":/icons/FEM_PostLineplot.svg", "ObjectsFem", "makePostLineplot"
)

post_visualization.register_extractor(
    "Lineplot",
    "LineplotFieldData",
    ":/icons/FEM_PostField.svg",
    "2D",
    "Field",
    "ObjectsFem",
    "makePostLineplotFieldData",
)

post_visualization.register_extractor(
    "Lineplot",
    "LineplotIndexOverFrames",
    ":/icons/FEM_PostIndex.svg",
    "2D",
    "Index",
    "ObjectsFem",
    "makePostLineplotIndexOverFrames",
)


# Implementation
# ##############


def is_lineplot_extractor(obj):

    if not base_fempostextractors.is_extractor_object(obj):
        return False

    if not hasattr(obj.Proxy, "VisualizationType"):
        return False

    return obj.Proxy.VisualizationType == "Lineplot"


class PostLineplotFieldData(post_extract2D.PostFieldData2D):
    """
    A 2D Field extraction for lineplot.
    """

    VisualizationType = "Lineplot"


class PostLineplotIndexOverFrames(post_extract2D.PostIndexOverFrames2D):
    """
    A 2D index extraction for lineplot.
    """

    VisualizationType = "Lineplot"


class PostLineplot(base_fempostvisualizations.PostVisualization):
    """
    A post processing plot for showing extracted data as line plots
    """

    VisualizationType = "Lineplot"



from Base.Metadata import export
from Base.BaseClass import BaseClass
from typing import Any, List, Dict

@export(
    Include="Gui/Workbench.h",
)
class Workbench(BaseClass):
    """
    This is the base class for workbenches

    Author: Werner Mayer (wmayer@users.sourceforge.net)
    Licence: LGPL
    """

    def name(self) -> str:
        """
        Return the workbench name
        """
        ...

    def activate(self) -> None:
        """
        Activate this workbench
        """
        ...

    def listToolbars(self) -> List[Any]:
        """
        Show a list of all toolbars
        """
        ...

    def getToolbarItems(self) -> Dict[Any, Any]:
        """
        Show a dict of all toolbars and their commands
        """
        ...

    def listCommandbars(self) -> List[Any]:
        """
        Show a list of all command bars
        """
        ...

    def listMenus(self) -> List[Any]:
        """
        Show a list of all menus
        """
        ...

    @staticmethod
    def reloadActive() -> None:
        """
        Reload the active workbench after changing menus or toolbars
        """
        ...




import FreeCAD
import codecs
import os
import unittest
from .TechDrawTestUtilities import createPageWithSVGTemplate


class DrawViewSymbolTest(unittest.TestCase):
    def setUp(self):
        """Creates a page"""
        FreeCAD.newDocument("TDAnno")
        FreeCAD.setActiveDocument("TDAnno")
        FreeCAD.ActiveDocument = FreeCAD.getDocument("TDAnno")
        self.page = createPageWithSVGTemplate()

    def tearDown(self):
        FreeCAD.closeDocument("TDAnno")

    def testMakeSymbol(self):
        """Tests if an symbol can be added to page"""
        sym = FreeCAD.ActiveDocument.addObject("TechDraw::DrawViewSymbol", "TestSymbol")
        path = os.path.dirname(os.path.abspath(__file__))
        symbolFileSpec = path + "/TestSymbol.svg"
        f = codecs.open(symbolFileSpec, "r", encoding="utf-8")
        svg = f.read()
        f.close()
        sym.Symbol = svg
        self.page.addView(sym)
        sym.X = 220.0
        sym.Y = 150.0

        FreeCAD.ActiveDocument.recompute()

        self.assertTrue("Up-to-date" in sym.State)

    def testNonAsciiSymbol(self):
        """Tests if a Non-Ascii symbol can be added to page"""
        sym = FreeCAD.ActiveDocument.addObject(
            "TechDraw::DrawViewSymbol", "NonAsciiSymbol"
        )
        path = os.path.dirname(os.path.abspath(__file__))
        symbolFileSpec = path + "/TestNonAsciiSymbol.svg"
        f = codecs.open(symbolFileSpec, "r", encoding="utf-8")
        svg = f.read()
        f.close()
        sym.Symbol = svg
        self.page.addView(sym)
        sym.X = 220.0
        sym.Y = 150.0

        FreeCAD.ActiveDocument.recompute()

        self.assertTrue("Up-to-date" in sym.State)



if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: LGPL-2.1-or-later
# --------------------------------------------------------------------------
#                                                                          *
#    Copyright (c) 2026 Chris Jones github.com/ipatch                      *
#                                                                          *
#    This file is part of FreeCAD.                                         *
#                                                                          *
#    FreeCAD is free software: you can redistribute it and/or modify it    *
#    under the terms of the GNU Lesser General Public License as           *
#    published by the Free Software Foundation, either version 2.1 of the  *
#    License, or (at your option) any later version.                       *
#                                                                          *
#    FreeCAD is distributed in the hope that it will be useful, but        *
#    WITHOUT ANY WARRANTY; without even the implied warranty of            *
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
#    Lesser General Public License for more details.                       *
#                                                                          *
#    You should have received a copy of the GNU Lesser General Public      *
#    License along with FreeCAD. If not, see                               *
#    <https://www.gnu.org/licenses/>.                                      *
#                                                                          *
# --------------------------------------------------------------------------

"""Unit tests for Tree widget 'Select all instances' functionality.

See GitHub issue #26478.
- https://github.com/FreeCAD/FreeCAD/issues/26478

To run tests:
    FreeCAD -t TestTreeSelection.TestSelectAllInstances
"""

import unittest
import FreeCAD
import FreeCADGui
from FreeCADGui import Selection

import Draft

from PySide6 import QtWidgets


class TestSelectAllInstances(unittest.TestCase):
    """Test cases for 'Select all instances' tree functionality.

    These tests verify that the Std_TreeSelectAllInstances command
    correctly selects all instances of an object, including those
    that appear under link-like objects (e.g., Part::Mirroring).

    See GitHub issue #26478.
    """

    def setUp(self):
        """Set up test document with objects."""
        self.doc = FreeCAD.newDocument("TestSelectAllInstances")
        FreeCADGui.ActiveDocument = FreeCADGui.getDocument(self.doc.Name)

    def tearDown(self):
        """Clean up test document."""
        Selection.clearSelection()
        FreeCAD.closeDocument(self.doc.Name)

    def _get_tree_widget(self):
        """Get the main tree widget."""
        mw = FreeCADGui.getMainWindow()
        for widget in mw.findChildren(QtWidgets.QTreeWidget):
            if widget.topLevelItemCount() > 0:
                return widget
        return None

    def _count_tree_selections_by_name(self, name):
        """Count how many tree items with given name are selected."""
        tree = self._get_tree_widget()
        if not tree:
            return 0
        selected = tree.selectedItems()
        return len([item for item in selected if item.text(0) == name])

    def test_select_all_instances_with_mirror(self):
        """Test that 'Select all instances' selects objects under Mirror objects.

        test for issue #26478: When an object exists in a Group
        and is also referenced by Mirror objects, 'Select all instances'
        should select all instances including those under the Mirror objects.
        """
        # Create a Draft Rectangle (matches original Part::Part2DObjectPython)
        rect = Draft.make_rectangle(10, 10)
        rect.Label = "Rectangle"

        # Create a group containing the rectangle and mirrors
        group = self.doc.addObject("App::DocumentObjectGroup", "Group")
        group.addObject(rect)

        # Create mirror objects that reference the rectangle
        # Use same naming as original document
        mirror1 = self.doc.addObject("Part::Mirroring", "Rectangle (mirrored)")
        mirror1.Source = rect
        mirror1.Normal = FreeCAD.Vector(1, 0, 0)
        mirror1.Base = FreeCAD.Vector(20, 0, 0)
        group.addObject(mirror1)

        mirror2 = self.doc.addObject("Part::Mirroring", "Rectangle (mirrored)001")
        mirror2.Source = rect
        mirror2.Normal = FreeCAD.Vector(0, 1, 0)
        mirror2.Base = FreeCAD.Vector(0, 20, 0)
        group.addObject(mirror2)

        self.doc.recompute()
        FreeCADGui.updateGui()

        self._debug_print_document_structure()

        # Get tree widget
        tree = self._get_tree_widget()
        self.assertIsNotNone(tree, "Could not find tree widget")

        # Clear any existing selection
        tree.clearSelection()
        Selection.clearSelection()

        # Select the rectangle via Gui.Selection (this will select it in the tree)
        Selection.addSelection(self.doc.Name, rect.Name)
        FreeCADGui.updateGui()

        # Run the "Select all instances" command
        FreeCADGui.runCommand("Std_TreeSelectAllInstances", 0)
        FreeCADGui.updateGui()

        # Count selected Rectangle items in tree
        count = self._count_tree_selections_by_name("Rectangle")

        # We expect 3 instances:
        # 1. Rectangle directly in Group
        # 2. Rectangle under Rectangle (mirrored)
        # 3. Rectangle under Rectangle (mirrored)001
        self.assertEqual(
            count,
            3,
            f"Expected 3 'Rectangle' instances selected in tree, got {count}. "
            f"See GitHub issue #26478.",
        )

    def test_select_all_instances_with_cuts(self):
        """Test that 'Select all instances' selects objects used in Part::Cut operations.

        When the same object is used as the Tool in multiple Cut operations,
        'Select all instances' should select all instances.

        loosely taken from the below freecad wiki page,
        https://wiki.freecad.org/Std_TreeSelectAllInstances
        """
        # Create the shared cube (tool for all cuts)
        cube = self.doc.addObject("Part::Box", "Cube")
        cube.Length = 10
        cube.Width = 10
        cube.Height = 10

        # Create base shapes
        cylinder = self.doc.addObject("Part::Cylinder", "Cylinder")
        cylinder.Radius = 8
        cylinder.Height = 15

        sphere = self.doc.addObject("Part::Sphere", "Sphere")
        sphere.Radius = 10

        cone = self.doc.addObject("Part::Cone", "Cone")
        cone.Radius1 = 10
        cone.Radius2 = 0
        cone.Height = 15

        # Create cuts - all using the same Cube as the tool
        cut1 = self.doc.addObject("Part::Cut", "Cut")
        cut1.Base = cylinder
        cut1.Tool = cube

        cut2 = self.doc.addObject("Part::Cut", "Cut001")
        cut2.Base = sphere
        cut2.Tool = cube

        cut3 = self.doc.addObject("Part::Cut", "Cut002")
        cut3.Base = cone
        cut3.Tool = cube

        self.doc.recompute()
        FreeCADGui.updateGui()

        self._debug_print_document_structure()

        # Get tree widget
        tree = self._get_tree_widget()
        self.assertIsNotNone(tree, "Could not find tree widget")

        # Clear any existing selection
        tree.clearSelection()
        Selection.clearSelection()

        # Select the cube
        Selection.addSelection(self.doc.Name, cube.Name)
        FreeCADGui.updateGui()

        # Run the "Select all instances" command
        FreeCADGui.runCommand("Std_TreeSelectAllInstances", 0)
        FreeCADGui.updateGui()

        # Count selected Cube items in tree
        count = self._count_tree_selections_by_name("Cube")

        self._debug_print_tree_selection(tree, count)

        # We expect 3 instances:
        # 1. Cube under Cut
        # 2. Cube under Cut001
        # 3. Cube under Cut002
        self.assertEqual(
            count,
            3,
            f"Expected 3 'Cube' instances selected in tree, got {count}.",
        )

    def test_select_all_instances_basic(self):
        """Test basic 'Select all instances' with single object in group."""
        # Create a simple box
        box = self.doc.addObject("Part::Box", "Box")
        box.Length = 10
        box.Width = 10
        box.Height = 10

        # Create a group containing the box
        group = self.doc.addObject("App::DocumentObjectGroup", "Group")
        group.addObject(box)

        self.doc.recompute()
        FreeCADGui.updateGui()

        # Get tree widget
        tree = self._get_tree_widget()
        self.assertIsNotNone(tree, "Could not find tree widget")

        # Clear and select
        tree.clearSelection()
        Selection.clearSelection()
        Selection.addSelection(self.doc.Name, box.Name)
        FreeCADGui.updateGui()

        # Run command
        FreeCADGui.runCommand("Std_TreeSelectAllInstances", 0)
        FreeCADGui.updateGui()

        # Should have at least 1 selection
        count = self._count_tree_selections_by_name("Box")
        self.assertGreaterEqual(
            count, 1, f"Expected at least 1 'Box' instance selected, got {count}."
        )



from Base.Metadata import export, constmethod
from Base.PyObjectBase import PyObjectBase
from Base.Vector import Vector
from TopoShape import TopoShape
from typing import overload

@export(
    PythonName="Part.BRepOffsetAPI_MakePipeShell",
    Include="BRepOffsetAPI_MakePipeShell.hxx",
    Constructor=True,
    Delete=True,
)
class BRepOffsetAPI_MakePipeShell(PyObjectBase):
    """
    Low level API to create a PipeShell using OCC API

    Ref: https://dev.opencascade.org/doc/refman/html/class_b_rep_offset_a_p_i___make_pipe_shell.html

    Author: Werner Mayer (wmayer[at]users.sourceforge.net)
    Licence: LGPL
    """

    def setFrenetMode(self, mode: bool) -> None:
        """
        setFrenetMode(True|False)
        Sets a Frenet or a CorrectedFrenet trihedron to perform the sweeping.
        True  = Frenet
        False = CorrectedFrenet
        """
        ...

    def setTrihedronMode(self, point: Vector, direction: Vector) -> None:
        """
        setTrihedronMode(point,direction)
        Sets a fixed trihedron to perform the sweeping.
        All sections will be parallel.
        """
        ...

    def setBiNormalMode(self, direction: Vector) -> None:
        """
        setBiNormalMode(direction)
        Sets a fixed BiNormal direction to perform the sweeping.
        Angular relations between the section(s) and the BiNormal direction will be constant.
        """
        ...

    def setSpineSupport(self, shape: TopoShape) -> None:
        """
        setSpineSupport(shape)
        Sets support to the spine to define the BiNormal of the trihedron, like the normal to the surfaces.
        Warning: To be effective, Each edge of the spine must have an representation on one face of SpineSupport.
        """
        ...

    def setAuxiliarySpine(
        self, wire: TopoShape, CurvilinearEquivalence: bool, TypeOfContact: int
    ) -> None:
        """
        setAuxiliarySpine(wire, CurvilinearEquivalence, TypeOfContact)
        Sets an auxiliary spine to define the Normal.

        CurvilinearEquivalence = bool
        For each Point of the Spine P, an Point Q is evalued on AuxiliarySpine.
        If CurvilinearEquivalence=True Q split AuxiliarySpine with the same length ratio than P split Spine.

        * OCC >= 6.7
        TypeOfContact = long
        0: No contact
        1: Contact
        2: Contact On Border (The auxiliary spine becomes a boundary of the swept surface)
        """
        ...

    @overload
    def add(
        self, Profile: TopoShape, *, WithContact: bool = False, WithCorrection: bool = False
    ) -> None: ...
    @overload
    def add(
        self,
        Profile: TopoShape,
        Location: TopoShape,
        *,
        WithContact: bool = False,
        WithCorrection: bool = False
    ) -> None: ...
    def add(self, **kwargs) -> None:
        """
        add(shape Profile, bool WithContact=False, bool WithCorrection=False)
        add(shape Profile, vertex Location, bool WithContact=False, bool WithCorrection=False)
        Adds the section Profile to this framework.
        First and last sections may be punctual, so the shape Profile may be both wire and vertex.
        If WithContact is true, the section is translated to be in contact with the spine.
        If WithCorrection is true, the section is rotated to be orthogonal to the spine tangent in the correspondent point.
        """
        ...

    def remove(self, Profile: TopoShape) -> None:
        """
        remove(shape Profile)
        Removes the section Profile from this framework.
        """
        ...

    def isReady(self) -> bool:
        """
        isReady()
        Returns true if this tool object is ready to build the shape.
        """
        ...

    def getStatus(self) -> int:
        """
        getStatus()
        Get a status, when Simulate or Build failed.
        """
        ...

    def makeSolid(self) -> bool:
        """
        makeSolid()
        Transforms the sweeping Shell in Solid. If a propfile is not closed returns False.
        """
        ...

    def setTolerance(self, tol3d: float, boundTol: float, tolAngular: float) -> None:
        """
        setTolerance( tol3d, boundTol, tolAngular)
        Tol3d = 3D tolerance
        BoundTol = boundary tolerance
        TolAngular = angular tolerance
        """
        ...

    def setTransitionMode(self, mode: int) -> None:
        """
        0: BRepBuilderAPI_Transformed
        1: BRepBuilderAPI_RightCorner
        2: BRepBuilderAPI_RoundCorner
        """
        ...

    def firstShape(self) -> TopoShape:
        """
        firstShape()
        Returns the Shape of the bottom of the sweep.
        """
        ...

    def lastShape(self) -> TopoShape:
        """
        lastShape()
        Returns the Shape of the top of the sweep.
        """
        ...

    def build(self) -> None:
        """
        build()
        Builds the resulting shape.
        """
        ...

    def shape(self) -> TopoShape:
        """
        shape()
        Returns the resulting shape.
        """
        ...

    def generated(self, S: TopoShape) -> list[TopoShape]:
        """
        generated(shape S)
        Returns a list of new shapes generated from the shape S by the shell-generating algorithm.
        """
        ...

    def setMaxDegree(self, degree: int) -> None:
        """
        setMaxDegree(int degree)
        Define the maximum V degree of resulting surface.
        """
        ...

    def setMaxSegments(self, num: int) -> None:
        """
        setMaxSegments(int num)
        Define the maximum number of spans in V-direction on resulting surface.
        """
        ...

    def setForceApproxC1(self, flag: bool) -> None:
        """
        setForceApproxC1(bool)
        Set the flag that indicates attempt to approximate a C1-continuous surface if a swept surface proved to be C0.
        """
        ...

    def simulate(self, nbsec: int) -> None:
        """
        simulate(int nbsec)
        Simulates the resulting shape by calculating the given number of cross-sections.
        """
        ...



from Base.Metadata import export, constmethod
from Base.Persistence import Persistence
from Base.Vector import Vector
from typing import Final, Tuple

@export(
    Include="Mod/Sketcher/App/Sketch.h",
    FatherInclude="Base/PersistencePy.h",
    Constructor=True,
)
class Sketch(Persistence):
    """
    With this objects you can handle constraint sketches

    Author: Juergen Riegel (FreeCAD@juergen-riegel.net)
    Licence: LGPL
    """

    Constraint: Final[int] = 0
    """0: exactly constraint, -1 under-constraint, 1 over-constraint"""

    Conflicts: Final[Tuple] = ()
    """Tuple of conflicting constraints"""

    Redundancies: Final[Tuple] = ()
    """Tuple of redundant constraints"""

    Geometries: Final[Tuple] = ()
    """Tuple of all geometric elements in this sketch"""

    Shape: Final[object] = None
    """Resulting shape from the sketch geometry"""

    def solve(self) -> None:
        """
        Solve the actual set of geometry and constraints
        """
        ...

    def addGeometry(self) -> None:
        """
        Add a geometric object to the sketch
        """
        ...

    def addConstraint(self) -> None:
        """
        Add an constraint object to the sketch
        """
        ...

    def clear(self) -> None:
        """
        Clear the sketch
        """
        ...

    def moveGeometry(
        self, GeoIndex: int, PointPos: Vector, Vector: Vector, relative: bool = False
    ) -> None:
        """
        moveGeometry(GeoIndex,PointPos,Vector,[relative]) - move a given point (or curve)
        to another location.
        It moves the specified point (or curve) to the given location by adding some
        temporary weak constraints and solve the sketch.
        This method is mostly used to allow the user to drag some portions of the sketch
        in real time by e.g. the mouse and it works only for underconstrained portions of
        the sketch.
        The argument 'relative', if present, states if the new location is given
        relatively to the current one.
        """
        ...



from typing import Any, Final

from Base.Metadata import constmethod, export
from Data import object

@export(
    Father="ComplexGeoDataPy",
    Twin="PointKernel",
    TwinPointer="PointKernel",
    Include="Mod/Points/App/Points.h",
    Namespace="Points",
    FatherInclude="App/ComplexGeoDataPy.h",
    FatherNamespace="Data",
    Constructor=True,
)
class Points(object):
    """
    Points() -- Create an empty points object.

    This class allows one to manipulate the Points object by adding new points, deleting facets, importing from an STL file,
    transforming and much more.

    Author: Juergen Riegel (Juergen.Riegel@web.de)
    License: LGPL-2.1-or-later
    """

    @constmethod
    def copy(self) -> Any:
        """Create a copy of this points object"""
        ...

    def read(self) -> Any:
        """Read in a points object from file."""
        ...

    @constmethod
    def write(self) -> Any:
        """Write the points object into file."""
        ...

    @constmethod
    def writeInventor(self) -> Any:
        """Write the points in OpenInventor format to a string."""
        ...

    def addPoints(self) -> Any:
        """add one or more (list of) points to the object"""
        ...

    @constmethod
    def fromSegment(self) -> Any:
        """Get a new point object from a given segment"""
        ...

    @constmethod
    def fromValid(self) -> Any:
        """Get a new point object from points with valid coordinates (i.e. that are not NaN)"""
        ...
    CountPoints: Final[int]
    """Return the number of vertices of the points object."""

    Points: Final[list]
    """A collection of points
With this attribute it is possible to get access to the points of the object

for p in pnt.Points:
	print p"""



from typing import Any, Final

from Base.Metadata import constmethod, export

from App.ComplexGeoData import ComplexGeoData

@export(
    Include="Mod/Fem/App/FemMesh.h",
    Namespace="Fem",
    FatherInclude="App/ComplexGeoDataPy.h",
    FatherNamespace="Data",
    Constructor=True,
)
class FemMesh(ComplexGeoData):
    """
    FemMesh class

    Author: Juergen Riegel (Juergen.Riegel@web.de)
    License: LGPL-2.1-or-later
    """

    def setShape(self) -> Any:
        """Set the Part shape to mesh"""
        ...

    def compute(self) -> Any:
        """Update the internal mesh structure"""
        ...

    def addHypothesis(self) -> Any:
        """Add hypothesis"""
        ...

    def setStandardHypotheses(self) -> Any:
        """Set some standard hypotheses for the whole shape"""
        ...

    def addNode(self) -> Any:
        """Add a node by setting (x,y,z)."""
        ...

    def addEdge(self) -> Any:
        """Add an edge by setting two node indices."""
        ...

    def addEdgeList(self) -> Any:
        """Add list of edges by list of node indices and list of nodes per edge."""
        ...

    def addFace(self) -> Any:
        """Add a face by setting three node indices."""
        ...

    def addFaceList(self) -> Any:
        """Add list of faces by list of node indices and list of nodes per face."""
        ...

    def addQuad(self) -> Any:
        """Add a quad by setting four node indices."""
        ...

    def addVolume(self) -> Any:
        """Add a volume by setting an arbitrary number of node indices."""
        ...

    def addVolumeList(self) -> Any:
        """Add list of volumes by list of node indices and list of nodes per volume."""
        ...

    def read(self) -> Any:
        """
        Read in a various FEM mesh file formats.
                            read(file.endingToExportTo)
                            supported formats: DAT, INP, MED, STL, UNV, VTK, Z88"""
        ...

    @constmethod
    def write(self) -> Any:
        """
        Write out various FEM mesh file formats.
                            write(file.endingToExportTo)
                            supported formats: BDF, DAT, INP, MED, STL, UNV, VTK, Z88"""
        ...

    @constmethod
    def writeABAQUS(self, **kwargs) -> Any:
        """
        Write out as ABAQUS inp
                            writeABAQUS(file, int elemParam, bool groupParam, str volVariant, str faceVariant, str edgeVariant)

                            elemParam:
                                0: All elements
                                1: Highest elements only
                                2: FEM elements only (only edges not belonging to faces and faces not belonging to volumes)

                            groupParam:
                                True: Write group data
                                False: Do not write group data

                            volVariant: Volume elements
                                "standard": Tetra4 -> C3D4, Penta6 -> C3D6, Hexa8 -> C3D8, Tetra10 -> C3D10, Penta15 -> C3D15, Hexa20 -> C3D20
                                "reduced": Hexa8 -> C3D8R, Hexa20 -> C3D20R
                                "incompatible": Hexa8 -> C3D8I
                                "modified": Tetra10 -> C3D10T
                                "fluid": Tetra4 -> F3D4, Penta6 -> F3D6, Hexa8  -> F3D8

                            faceVariant: Face elements
                                "shell": Tria3 -> S3, Quad4 -> S4, Tria6 -> S6, Quad8 -> S8
                                "shell reduced": Tria3 -> S3, Quad4 -> S4R, Tria6 -> S6, Quad8 -> S8R
                                "membrane": Tria3 -> M3D3, Quad4 -> M3D4, Tria6 -> M3D6, Quad8 -> M3D8
                                "membrane reduced": Tria3 -> M3D3, Quad4 -> M3D4R, Tria6 -> M3D6, Quad8 -> M3D8R
                                "stress": Tria3 -> CPS3, Quad4 -> CPS4, Tria6 -> CPS6, Quad8 -> CPS8
                                "stress reduced": Tria3 -> CPS3, Quad4 -> CPS4R, Tria6 -> CPS6, Quad8 -> CPS8R
                                "strain": Tria3 -> CPE3, Quad4 -> CPE4, Tria6 -> CPE6, Quad8 -> CPE8
                                "strain reduced": Tria3 -> CPE3, Quad4 -> CPE4R, Tria6 -> CPE6, Quad8 -> CPE8R
                                "axisymmetric": Tria3 -> CAX3, Quad4 -> CAX4, Tria6 -> CAX6, Quad8 -> CAX8
                                "axisymmetric reduced": Tria3 -> CAX3, Quad4 -> CAX4R, Tria6 -> CAX6, Quad8 -> CAX8R

                            edgeVariant: Edge elements
                                "beam": Seg2 -> B31, Seg3 -> B32
                                "beam reduced": Seg2 -> B31R, Seg3 -> B32R
                                "truss": Seg2 -> T3D2, eg3 -> T3D3
                                "network": Seg3 -> D

                            Elements are selected according to CalculiX availability.
                            For example if volume variant "modified" is selected, Tetra10 mesh
                            elements are assigned to C3D10T and remain elements uses "standard".
                            Axisymmetric, plane strain and plane stress elements expect nodes in the plane z=0.
        """
        ...

    def setTransform(self) -> Any:
        """Use a Placement object to perform a translation or rotation"""
        ...

    @constmethod
    def copy(self) -> Any:
        """Make a copy of this FEM mesh."""
        ...

    @constmethod
    def getFacesByFace(self) -> Any:
        """Return a list of face IDs which belong to a TopoFace"""
        ...

    @constmethod
    def getEdgesByEdge(self) -> Any:
        """Return a list of edge IDs which belong to a TopoEdge"""
        ...

    @constmethod
    def getVolumesByFace(self) -> Any:
        """Return a dict of volume IDs and face IDs which belong to a TopoFace"""
        ...

    @constmethod
    def getccxVolumesByFace(self) -> Any:
        """Return a dict of volume IDs and ccx face numbers which belong to a TopoFace"""
        ...

    @constmethod
    def getNodeById(self) -> Any:
        """Get the node position vector by a Node-ID"""
        ...

    @constmethod
    def getNodesBySolid(self) -> Any:
        """Return a list of node IDs which belong to a TopoSolid"""
        ...

    @constmethod
    def getNodesByFace(self) -> Any:
        """Return a list of node IDs which belong to a TopoFace"""
        ...

    @constmethod
    def getNodesByEdge(self) -> Any:
        """Return a list of node IDs which belong to a TopoEdge"""
        ...

    @constmethod
    def getNodesByVertex(self) -> Any:
        """Return a list of node IDs which belong to a TopoVertex"""
        ...

    @constmethod
    def getElementNodes(self) -> Any:
        """Return a tuple of node IDs to a given element ID"""
        ...

    @constmethod
    def getNodeElements(self) -> Any:
        """Return a tuple of specific element IDs associated to a given node ID"""
        ...

    @constmethod
    def getGroupName(self) -> Any:
        """Return a string of group name to a given group ID"""
        ...

    @constmethod
    def getGroupElementType(self) -> Any:
        """Return a string of group element type to a given group ID"""
        ...

    @constmethod
    def getGroupElements(self) -> Any:
        """Return a tuple of ElementIDs to a given group ID"""
        ...

    @constmethod
    def addGroup(self) -> Any:
        """
        Add a group to mesh with specific name and type
                            addGroup(name, typestring, [id])
                            name: string
                            typestring: "All", "Node", "Edge", "Face", "Volume", "0DElement", "Ball"
                            id: int
                            Optional id is used to force specific id for group, but does
                            not work, yet."""
        ...

    @constmethod
    def addGroupElements(self) -> Any:
        """
        Add a tuple of ElementIDs to a given group ID
                            addGroupElements(groupid, list_of_elements)
                            groupid: int
                            list_of_elements: list of int
                            Notice that the elements have to be in the mesh."""
        ...

    @constmethod
    def removeGroup(self) -> Any:
        """
        Remove a group with a given group ID
                            removeGroup(groupid)
                            groupid: int
                            Returns boolean."""
        ...

    @constmethod
    def renameGroup(self) -> Any:
        """Rename a group with a given group ID
        renameGroup(id, name)
        groupid: int
        name: string"""
        ...

    @constmethod
    def getElementType(self) -> Any:
        """Return the element type of a given ID"""
        ...

    @constmethod
    def getIdByElementType(self) -> Any:
        """Return a tuple of IDs to a given element type"""
        ...
    Nodes: Final[dict]
    """Dictionary of Nodes by ID (int ID:Vector())"""

    NodeCount: Final[int]
    """Number of nodes in the Mesh."""

    Edges: Final[tuple]
    """Tuple of edge IDs"""

    EdgesOnly: Final[tuple]
    """Tuple of edge IDs which does not belong to any face (and thus not belong to any volume too)"""

    EdgeCount: Final[int]
    """Number of edges in the Mesh."""

    Faces: Final[tuple]
    """Tuple of face IDs"""

    FacesOnly: Final[tuple]
    """Tuple of face IDs which does not belong to any volume"""

    FaceCount: Final[int]
    """Number of Faces in the Mesh."""

    TriangleCount: Final[int]
    """Number of Triangles in the Mesh."""

    QuadrangleCount: Final[int]
    """Number of Quadrangles in the Mesh."""

    PolygonCount: Final[int]
    """Number of Quadrangles in the Mesh."""

    Volumes: Final[tuple]
    """Tuple of volume IDs"""

    VolumeCount: Final[int]
    """Number of Volumes in the Mesh."""

    TetraCount: Final[int]
    """Number of Tetras in the Mesh."""

    HexaCount: Final[int]
    """Number of Hexas in the Mesh."""

    PyramidCount: Final[int]
    """Number of Pyramids in the Mesh."""

    PrismCount: Final[int]
    """Number of Prisms in the Mesh."""

    PolyhedronCount: Final[int]
    """Number of Polyhedrons in the Mesh."""

    SubMeshCount: Final[int]
    """Number of SubMeshs in the Mesh."""

    GroupCount: Final[int]
    """Number of Groups in the Mesh."""

    Groups: Final[tuple]
    """Tuple of Group IDs."""

    Volume: Final[Any]
    """Volume of the mesh."""



# SPDX-License-Identifier: LGPL-2.1-or-later

import FreeCAD

vec2 = FreeCAD.Base.Vector2d
import Part

import unittest


class Geom2dTests(unittest.TestCase):

    def test_toShape(self):
        surf = Part.Cylinder()

        p1 = vec2(1.0, -1.0)
        p2 = vec2(2.0, -1.0)

        l12 = Part.Geom2d.Line2dSegment(p1, p2)

        e1 = l12.toShape()
        e2 = l12.toShape(surf)
        self.assertNotEqual(e1.curveOnSurface(0), None)
        self.assertNotEqual(e2.curveOnSurface(0), None)

    def test_insertKnot(self):
        with self.assertRaises(TypeError):
            curve = Part.Geom2d.BSplineCurve2d()
            curve.insertKnot(0.5, 1, 0.01, 2)



# ***************************************************************************
# *   Copyright (c) 2025 Stefan Tröger <stefantroeger@gmx.net>              *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD FEM post extractor object task panel"
__author__ = "Stefan Tröger"
__url__ = "https://www.freecad.org"

## @package task_post_extractor
#  \ingroup FEM
#  \brief universal task dialog for extractor objects.

from PySide import QtCore, QtGui

from . import base_fempostpanel


class _ExtractorTaskPanel(base_fempostpanel._BasePostTaskPanel):
    """
    The TaskPanel for editing properties extractor objects. The actual UI is
    provided by the viewproviders. This allows using a universal task panel
    """

    def __init__(self, obj):
        super().__init__(obj)

        # form is used to display individual task panels
        app = obj.ViewObject.Proxy.get_app_edit_widget(self)
        app.setWindowTitle("Data Extraction")
        app.setWindowIcon(obj.ViewObject.Icon)
        view = obj.ViewObject.Proxy.get_view_edit_widget(self)
        view.setWindowTitle("Visualization Settings")
        view.setWindowIcon(obj.ViewObject.Icon)

        self.form = [app, view]
